import django_filters

from waldur_core.core import filters as core_filters
from waldur_core.structure import filters as structure_filters

from . import models


class ImageFilter(structure_filters.ServicePropertySettingsFilter):
    class Meta(structure_filters.ServicePropertySettingsFilter.Meta):
        model = models.Image


class LocationFilter(structure_filters.ServicePropertySettingsFilter):
    class Meta(structure_filters.ServicePropertySettingsFilter.Meta):
        model = models.Location


class SizeFilter(structure_filters.ServicePropertySettingsFilter):
    class Meta(structure_filters.ServicePropertySettingsFilter.Meta):
        model = models.Size


class BaseResourceGroupFilter(structure_filters.BaseResourceFilter):
    resource_group = core_filters.URLFilter(
        view_name='azure-resource-group-detail',
        name='resource_group__uuid'
    )
    resource_group_uuid = django_filters.UUIDFilter(name='resource_group__uuid')


class VirtualMachineFilter(BaseResourceGroupFilter):
    class Meta(BaseResourceGroupFilter.Meta):
        model = models.VirtualMachine


class PublicIPFilter(BaseResourceGroupFilter):
    class Meta(BaseResourceGroupFilter.Meta):
        model = models.PublicIP


class SQLServerFilter(BaseResourceGroupFilter):
    class Meta(BaseResourceGroupFilter.Meta):
        model = models.SQLServer


class SQLDatabaseFilter(BaseResourceGroupFilter):
    class Meta(BaseResourceGroupFilter.Meta):
        model = models.SQLDatabase

    server = core_filters.URLFilter(
        view_name='azure-server-detail',
        name='server__uuid'
    )
    server_uuid = django_filters.UUIDFilter(name='server__uuid')



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-12-04 12:25
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('structure', '0004_customer_homepage'),
    ]

    operations = [
        migrations.AddField(
            model_name='customer',
            name='domain',
            field=models.CharField(blank=True, max_length=255),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-09-03 08:29
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0025_offering_plan_data'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='offering',
            name='type',
        ),
        migrations.AlterField(
            model_name='offering',
            name='template',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='support.OfferingTemplate'),
        ),
    ]



import django.dispatch

# This signal allows to implement custom quota recalculation
# without introducing circular dependency between core quotas application and plugins.
# It is called when recalculatequotas management command is called.
recalculate_quotas = django.dispatch.Signal()



from django.core.exceptions import ObjectDoesNotExist
from django.core.management.base import CommandError

from waldur_core.core.utils import DryRunCommand
from waldur_core.structure.models import Customer
from waldur_mastermind.marketplace.models import Category
from waldur_mastermind.marketplace_openstack import utils


class Command(DryRunCommand):
    help = """Import OpenStack service settings as marketplace offerings."""

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument('--customer', dest='customer_uuid', required=True,
                            help='Default customer argument is used for shared service setting.')

        parser.add_argument('--require-templates', action='store_true',
                            help='Skip service settings without package template.')

    def handle(self, customer_uuid, dry_run, require_templates, *args, **options):
        try:
            customer = Customer.objects.get(uuid=customer_uuid)
        except ObjectDoesNotExist:
            raise CommandError('A customer is not found.')

        try:
            offerings_counter, plans_counter = utils.import_openstack_service_settings(
                customer, dry_run, require_templates)
            self.stdout.write(self.style.SUCCESS('%s offerings have been created.' % offerings_counter))
            self.stdout.write(self.style.SUCCESS('%s plans have been created.' % plans_counter))
        except Category.DoesNotExist:
            raise CommandError('Please ensure that WALDUR_MARKETPLACE_OPENSTACK.TENANT_CATEGORY_UUID '
                               'setting has valid value.')



from django.apps import AppConfig
from django.conf import settings
from django.db.models import signals


def get_secret_attributes():
    if not settings.WALDUR_OPENSTACK['TENANT_CREDENTIALS_VISIBLE']:
        return 'user_username', 'user_password'


class MarketplaceOpenStackConfig(AppConfig):
    name = 'waldur_mastermind.marketplace_openstack'
    verbose_name = 'Marketplace OpenStack'

    def ready(self):
        from waldur_core.structure import models as structure_models
        from waldur_core.structure import signals as structure_signals
        from waldur_openstack.openstack import models as openstack_models
        from waldur_openstack.openstack.apps import OpenStackConfig
        from waldur_openstack.openstack_tenant import models as tenant_models
        from waldur_mastermind.marketplace import models as marketplace_models
        from waldur_mastermind.marketplace import handlers as marketplace_handlers
        from waldur_mastermind.marketplace.plugins import manager
        from waldur_mastermind.marketplace.plugins import Component

        from . import (
            handlers, processors,
            INSTANCE_TYPE, VOLUME_TYPE, PACKAGE_TYPE,
            RAM_TYPE, CORES_TYPE, STORAGE_TYPE,
        )

        signals.post_save.connect(
            handlers.create_offering_from_tenant,
            sender=openstack_models.Tenant,
            dispatch_uid='waldur_mastermind.marketpace_openstack.create_offering_from_tenant',
        )

        signals.pre_delete.connect(
            handlers.archive_offering,
            sender=structure_models.ServiceSettings,
            dispatch_uid='waldur_mastermind.marketpace_openstack.archive_offering',
        )

        resource_models = (tenant_models.Instance, tenant_models.Volume, openstack_models.Tenant)
        marketplace_handlers.connect_resource_metadata_handlers(*resource_models)
        marketplace_handlers.connect_resource_handlers(*resource_models)

        signals.post_save.connect(
            handlers.create_template_for_plan,
            sender=marketplace_models.Plan,
            dispatch_uid='waldur_mastermind.marketpace_openstack.create_template_for_plan',
        )

        signals.post_save.connect(
            handlers.synchronize_plan_component,
            sender=marketplace_models.PlanComponent,
            dispatch_uid='waldur_mastermind.marketpace_openstack.synchronize_plan_component',
        )

        FIXED = marketplace_models.OfferingComponent.BillingTypes.FIXED
        manager.register(offering_type=PACKAGE_TYPE,
                         create_resource_processor=processors.PackageCreateProcessor,
                         update_resource_processor=processors.PackageUpdateProcessor,
                         delete_resource_processor=processors.PackageDeleteProcessor,
                         components=(
                             Component(type=RAM_TYPE, name='RAM', measured_unit='GB', billing_type=FIXED),
                             Component(type=CORES_TYPE, name='Cores', measured_unit='cores', billing_type=FIXED),
                             Component(type=STORAGE_TYPE, name='Storage', measured_unit='GB', billing_type=FIXED),
                         ),
                         service_type=OpenStackConfig.service_name,
                         secret_attributes=get_secret_attributes)

        manager.register(offering_type=INSTANCE_TYPE,
                         create_resource_processor=processors.InstanceCreateProcessor,
                         delete_resource_processor=processors.InstanceDeleteProcessor)

        manager.register(offering_type=VOLUME_TYPE,
                         create_resource_processor=processors.VolumeCreateProcessor,
                         delete_resource_processor=processors.VolumeDeleteProcessor)

        signals.post_save.connect(
            handlers.synchronize_volume_metadata,
            sender=tenant_models.Volume,
            dispatch_uid='waldur_mastermind.marketpace_openstack.synchronize_volume_metadata',
        )

        signals.post_save.connect(
            handlers.synchronize_instance_metadata,
            sender=tenant_models.Instance,
            dispatch_uid='waldur_mastermind.marketpace_openstack.synchronize_instance_metadata',
        )

        signals.post_save.connect(
            handlers.synchronize_internal_ips,
            sender=tenant_models.InternalIP,
            dispatch_uid='waldur_mastermind.marketpace_openstack.synchronize_internal_ips',
        )

        signals.post_save.connect(
            handlers.synchronize_floating_ips,
            sender=tenant_models.FloatingIP,
            dispatch_uid='waldur_mastermind.marketpace_openstack.synchronize_floating_ips',
        )

        signals.post_save.connect(
            handlers.create_resource_of_volume_if_instance_created,
            sender=marketplace_models.Resource,
            dispatch_uid='waldur_mastermind.marketpace_openstack.'
                         'create_resource_of_volume_if_instance_created',
        )

        for model in [tenant_models.Instance, tenant_models.Volume]:
            structure_signals.resource_imported.connect(
                handlers.create_marketplace_resource_for_imported_resources,
                sender=model,
                dispatch_uid='waldur_mastermind.marketpace_openstack.'
                             'create_resource_for_imported_%s' % model,
            )

        signals.post_save.connect(
            handlers.import_resource_metadata_when_resource_is_created,
            sender=marketplace_models.Resource,
            dispatch_uid='waldur_mastermind.marketpace_openstack.'
                         'import_resource_metadata_when_resource_is_created',
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('openstack', '0005_ipmapping'),
    ]

    operations = [
        migrations.CreateModel(
            name='DRBackupRestoration',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False)),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('runtime_state', models.CharField(max_length=150, verbose_name='runtime state', blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='VolumeBackupRecord',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('service', models.CharField(max_length=200)),
                ('details', waldur_core.core.fields.JSONField(blank=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='VolumeBackupRestoration',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False)),
                ('uuid', waldur_core.core.fields.UUIDField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='drbackup',
            name='metadata',
            field=waldur_core.core.fields.JSONField(help_text='Information about instance that will be used on restoration', blank=True),
        ),
        migrations.AddField(
            model_name='volumebackup',
            name='size',
            field=models.PositiveIntegerField(default=0, help_text='Size of source volume in MiB'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='snapshot',
            name='source_volume',
            field=models.ForeignKey(related_name='snapshots', on_delete=django.db.models.deletion.SET_NULL, to='openstack.Volume', null=True),
        ),
        migrations.AlterField(
            model_name='volumebackup',
            name='metadata',
            field=waldur_core.core.fields.JSONField(help_text='Information about volume that will be used on restoration', blank=True),
        ),
        migrations.AddField(
            model_name='volumebackuprestoration',
            name='mirorred_volume_backup',
            field=models.ForeignKey(related_name='+', on_delete=django.db.models.deletion.SET_NULL, to='openstack.VolumeBackup', null=True),
        ),
        migrations.AddField(
            model_name='volumebackuprestoration',
            name='tenant',
            field=models.ForeignKey(related_name='volume_backup_restorations', to='openstack.Tenant'),
        ),
        migrations.AddField(
            model_name='volumebackuprestoration',
            name='volume',
            field=models.OneToOneField(related_name='+', to='openstack.Volume'),
        ),
        migrations.AddField(
            model_name='volumebackuprestoration',
            name='volume_backup',
            field=models.ForeignKey(related_name='restorations', to='openstack.VolumeBackup'),
        ),
        migrations.AddField(
            model_name='drbackuprestoration',
            name='dr_backup',
            field=models.ForeignKey(related_name='restorations', to='openstack.DRBackup'),
        ),
        migrations.AddField(
            model_name='drbackuprestoration',
            name='flavor',
            field=models.ForeignKey(related_name='+', to='openstack.Flavor'),
        ),
        migrations.AddField(
            model_name='drbackuprestoration',
            name='instance',
            field=models.OneToOneField(related_name='+', to='openstack.Instance'),
        ),
        migrations.AddField(
            model_name='drbackuprestoration',
            name='tenant',
            field=models.ForeignKey(related_name='+', to='openstack.Tenant', help_text='Tenant for instance restoration'),
        ),
        migrations.AddField(
            model_name='drbackuprestoration',
            name='volume_backup_restorations',
            field=models.ManyToManyField(related_name='_drbackuprestoration_volume_backup_restorations_+', to='openstack.VolumeBackupRestoration'),
        ),
        migrations.AddField(
            model_name='volumebackup',
            name='record',
            field=models.ForeignKey(related_name='volume_backups', on_delete=django.db.models.deletion.SET_NULL, to='openstack.VolumeBackupRecord', null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-12-08 09:34
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('invoices', '0025_servicedowntime'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='_file',
            field=models.TextField(blank=True, editable=False),
        ),
    ]



from django.apps import AppConfig
from django.db.models import signals


class DigitalOceanConfig(AppConfig):
    name = 'waldur_digitalocean'
    verbose_name = 'Waldur DigitalOcean'
    service_name = 'DigitalOcean'
    is_public_service = True

    def ready(self):
        from waldur_core.core import models as core_models
        from waldur_core.structure import SupportedServices, signals as structure_signals, models as structure_models

        from . import handlers
        from .backend import DigitalOceanBackend
        SupportedServices.register_backend(DigitalOceanBackend)

        for model in (structure_models.Project, structure_models.Customer):
            structure_signals.structure_role_revoked.connect(
                handlers.remove_ssh_keys_from_service,
                sender=model,
                dispatch_uid=('waldur_digitalocean.handlers.remove_ssh_keys_from_service__%s'
                              % model.__name__),
            )

        signals.pre_delete.connect(
            handlers.remove_ssh_key_from_service_settings_on_deletion,
            sender=core_models.SshPublicKey,
            dispatch_uid='waldur_digitalocean.handlers.remove_ssh_key_from_service_settings_on_deletion',
        )



from __future__ import unicode_literals

from django.conf.urls import url
from django.contrib import admin
from django.forms import ModelForm, ModelChoiceField
from django.http import HttpResponse
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy as _

from waldur_core.core import admin as core_admin
from waldur_core.core.admin_filters import RelatedOnlyDropdownFilter
from waldur_mastermind.packages import models as package_models

from . import executors, models, tasks


class GenericItemInline(core_admin.UpdateOnlyModelAdmin, admin.TabularInline):
    model = models.GenericInvoiceItem
    readonly_fields = (
        'name', 'price', 'unit_price', 'unit', 'start', 'end',
        'project_name', 'project_uuid', 'product_code', 'article_code',
        'format_details', 'quantity'
    )
    exclude = ('details', 'project', 'content_type', 'object_id')

    def format_details(self, obj):
        return core_admin.format_json_field(obj.details)

    format_details.allow_tags = True
    format_details.short_description = _('Details')


class InvoiceAdmin(core_admin.ExtraActionsMixin,
                   core_admin.UpdateOnlyModelAdmin,
                   admin.ModelAdmin):
    inlines = [GenericItemInline]
    fields = ['tax_percent', 'invoice_date', 'customer', 'state', 'total', 'year', 'month', 'pdf_file']
    readonly_fields = ('customer', 'total', 'year', 'month', 'pdf_file')
    list_display = ('customer', 'total', 'year', 'month', 'state')
    list_filter = ('state', 'customer')
    search_fields = ('customer__name', 'uuid')
    actions = ('create_pdf',)

    class CreatePDFAction(core_admin.ExecutorAdminAction):
        executor = executors.InvoicePDFCreateExecutor
        short_description = _('Create PDF')

    create_pdf = CreatePDFAction()

    def get_urls(self):
        my_urls = [
            url(r'^(.+)/change/pdf_file/$', self.admin_site.admin_view(self.pdf_file_view)),
        ]
        return my_urls + super(InvoiceAdmin, self).get_urls()

    def pdf_file_view(self, request, pk=None):
        invoice = models.Invoice.objects.get(id=pk)
        file_response = HttpResponse(invoice.file, content_type='application/pdf')
        filename = invoice.get_filename()
        file_response['Content-Disposition'] = 'attachment; filename="{filename}"'.format(filename=filename)
        return file_response

    def pdf_file(self, obj):
        if not obj.file:
            return ''

        return format_html('<a href="./pdf_file">download</a>')

    pdf_file.short_description = "File"

    def get_extra_actions(self):
        return [
            self.send_invoice_report,
            self.update_current_cost,
            self.create_pdf_for_all,
        ]

    def send_invoice_report(self, request):
        tasks.send_invoice_report.delay()
        message = _('Invoice report task has been scheduled')
        self.message_user(request, message)
        return redirect(reverse('admin:invoices_invoice_changelist'))

    send_invoice_report.short_description = _('Send invoice report as CSV to email')

    def update_current_cost(self, request):
        tasks.update_invoices_current_cost.delay()
        message = _('Task has been scheduled.')
        self.message_user(request, message)
        return redirect(reverse('admin:invoices_invoice_changelist'))

    send_invoice_report.short_description = _('Update current cost for invoices')

    def create_pdf_for_all(self, request):
        tasks.create_pdf_for_all_invoices.delay()
        message = _('PDF creation has been scheduled')
        self.message_user(request, message)
        return redirect(reverse('admin:invoices_invoice_changelist'))

    create_pdf_for_all.name = _('Create PDF for all invoices')


class PackageChoiceField(ModelChoiceField):
    def label_from_instance(self, obj):
        return '%s > %s > %s' % (
            obj.tenant.service_project_link.project.customer,
            obj.tenant.service_project_link.project.name,
            obj.tenant.name
        )


class ServiceDowntimeForm(ModelForm):
    package = PackageChoiceField(
        queryset=package_models.OpenStackPackage.objects.order_by(
            'tenant__service_project_link__project__customer__name',
            'tenant__service_project_link__project__name',
            'tenant__name',
        )
    )


class ServiceDowntimeAdmin(admin.ModelAdmin):
    list_display = ('get_customer', 'get_project', 'get_name', 'start', 'end')
    list_display_links = ('get_name',)
    list_filter = (
        ('package__tenant__service_project_link__project__customer', RelatedOnlyDropdownFilter),
        ('package__tenant__service_project_link__project', RelatedOnlyDropdownFilter),
    )
    search_fields = ('package__tenant__name',)
    date_hierarchy = 'start'
    form = ServiceDowntimeForm

    def get_readonly_fields(self, request, obj=None):
        # Downtime record is protected from modifications
        if obj is not None:
            return self.readonly_fields + ('start', 'end', 'package')
        return self.readonly_fields

    def get_customer(self, downtime):
        return downtime.package.tenant.service_project_link.project.customer

    get_customer.short_description = _('Organization')
    get_customer.admin_order_field = 'package__tenant__service_project_link__project__customer'

    def get_project(self, downtime):
        return downtime.package.tenant.service_project_link.project

    get_project.short_description = _('Project')
    get_project.admin_order_field = 'package__tenant__service_project_link__project'

    def get_name(self, downtime):
        return downtime.package.tenant.name

    get_name.short_description = _('Resource')
    get_name.admin_order_field = 'package__tenant__name'


admin.site.register(models.Invoice, InvoiceAdmin)
admin.site.register(models.ServiceDowntime, ServiceDowntimeAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-01 11:40
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_azure', '0007_publicip'),
    ]

    operations = [
        migrations.AddField(
            model_name='networkinterface',
            name='public_ip',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='waldur_azure.PublicIP'),
        ),
    ]



from .executors import ProjectImportExecutor
from .log import event_logger
from .models import Issue


def import_project_issues(sender, instance, **kwargs):
    ProjectImportExecutor.execute(instance, updated_fields=None)


def log_issue_save(sender, instance, created=False, **kwargs):
    if created or instance.state == Issue.States.CREATING:
        # we skip logging on instance creation as backend_id/JIRA key is not known yet
        pass
    else:
        if instance.tracker.previous('state') == Issue.States.CREATING and \
                instance.state == Issue.States.OK:
            event_logger.jira_issue.info(
                'Issue {issue_key} has been created.',
                event_type='issue_creation_succeeded',
                event_context={
                    'issue': instance,
                })
        else:
            event_logger.jira_issue.info(
                'Issue {issue_key} has been updated.',
                event_type='issue_update_succeeded',
                event_context={
                    'issue': instance,
                })


def log_issue_delete(sender, instance, **kwargs):
    event_logger.jira_issue.info(
        'Issue {issue_key} has been deleted.',
        event_type='issue_deletion_succeeded',
        event_context={
            'issue': instance,
        })


def log_comment_save(sender, instance, created=False, **kwargs):
    if created:
        event_logger.jira_comment.info(
            'Comment for issue {issue_key} has been created.',
            event_type='comment_creation_succeeded',
            event_context={
                'comment': instance,
            })
    else:
        event_logger.jira_comment.info(
            'Comment for issue {issue_key} has been updated.',
            event_type='comment_update_succeeded',
            event_context={
                'comment': instance,
            })


def log_comment_delete(sender, instance, **kwargs):
    event_logger.jira_comment.info(
        'Comment for issue {issue_key} has been deleted.',
        event_type='comment_deletion_succeeded',
        event_context={
            'comment': instance,
        })



from __future__ import unicode_literals

import datetime
import logging

import jwt
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.db import transaction
from django.db.models import OuterRef, Subquery, Count, IntegerField
from django.utils.translation import ugettext_lazy as _
from rest_framework import exceptions as rf_exceptions
from rest_framework import serializers
from rest_framework.reverse import reverse

from waldur_core.core import serializers as core_serializers
from waldur_core.core import signals as core_signals
from waldur_core.core import utils as core_utils
from waldur_core.core.fields import NaturalChoiceField
from waldur_core.core.serializers import GenericRelatedField
from waldur_core.quotas.serializers import BasicQuotaSerializer
from waldur_core.structure import models as structure_models, SupportedServices
from waldur_core.structure import permissions as structure_permissions
from waldur_core.structure import serializers as structure_serializers
from waldur_core.structure.managers import filter_queryset_for_user
from waldur_core.structure.tasks import connect_shared_settings
from waldur_mastermind.common.serializers import validate_options
from waldur_mastermind.marketplace.utils import validate_order_item
from waldur_mastermind.support import serializers as support_serializers

from . import models, attribute_types, plugins, utils, permissions, tasks

logger = logging.getLogger(__name__)


class ServiceProviderSerializer(core_serializers.AugmentedSerializerMixin,
                                serializers.HyperlinkedModelSerializer):
    class Meta(object):
        model = models.ServiceProvider
        fields = (
            'url', 'uuid', 'created', 'description', 'enable_notifications',
            'customer', 'customer_name', 'customer_uuid', 'customer_image',
            'customer_abbreviation', 'customer_native_name',
        )
        related_paths = {
            'customer': ('uuid', 'name', 'native_name', 'abbreviation')
        }
        protected_fields = ('customer',)
        extra_kwargs = {
            'url': {'lookup_field': 'uuid', 'view_name': 'marketplace-service-provider-detail'},
            'customer': {'lookup_field': 'uuid'},
        }

    customer_image = serializers.ImageField(source='customer.image', read_only=True)

    def validate(self, attrs):
        if not self.instance:
            permissions.can_register_service_provider(self.context['request'], attrs['customer'])
        return attrs


class NestedAttributeOptionSerializer(serializers.ModelSerializer):
    class Meta(object):
        model = models.AttributeOption
        fields = ('key', 'title')


class NestedAttributeSerializer(serializers.ModelSerializer):
    options = NestedAttributeOptionSerializer(many=True)

    class Meta(object):
        model = models.Attribute
        fields = ('key', 'title', 'type', 'options', 'required', 'default')


class NestedSectionSerializer(serializers.ModelSerializer):
    attributes = NestedAttributeSerializer(many=True, read_only=True)

    class Meta(object):
        model = models.Section
        fields = ('key', 'title', 'attributes', 'is_standalone')


class NestedColumnSerializer(serializers.ModelSerializer):
    class Meta(object):
        model = models.CategoryColumn
        fields = ('index', 'title', 'attribute', 'widget')


class CategoryComponentSerializer(serializers.ModelSerializer):
    class Meta(object):
        model = models.CategoryComponent
        fields = ('type', 'name', 'description', 'measured_unit')


class CategorySerializer(core_serializers.AugmentedSerializerMixin,
                         core_serializers.RestrictedSerializerMixin,
                         serializers.HyperlinkedModelSerializer):
    offering_count = serializers.ReadOnlyField()
    sections = NestedSectionSerializer(many=True, read_only=True)
    columns = NestedColumnSerializer(many=True, read_only=True)
    components = CategoryComponentSerializer(many=True, read_only=True)

    @staticmethod
    def eager_load(queryset, request):
        offerings = models.Offering.objects \
            .filter(state=models.Offering.States.ACTIVE) \
            .filter(category=OuterRef('pk')) \
            .filter_for_user(request.user)

        allowed_customer_uuid = request.query_params.get('allowed_customer_uuid')
        if allowed_customer_uuid and core_utils.is_uuid_like(allowed_customer_uuid):
            offerings = offerings.filter_for_customer(allowed_customer_uuid)

        project_uuid = request.query_params.get('project_uuid')
        if project_uuid and core_utils.is_uuid_like(project_uuid):
            offerings = offerings.filter_for_project(project_uuid)

        offerings = offerings \
            .annotate(count=Count('*'))\
            .values('count')

        # Workaround for Django bug:
        # https://code.djangoproject.com/ticket/28296
        # It allows to remove extra GROUP BY clause from the subquery.
        offerings.query.group_by = []

        offering_count = Subquery(offerings[:1], output_field=IntegerField())
        queryset = queryset.annotate(offering_count=offering_count)
        return queryset.prefetch_related('sections', 'sections__attributes')

    class Meta(object):
        model = models.Category
        fields = ('url', 'uuid', 'title', 'description', 'icon', 'offering_count',
                  'sections', 'columns', 'components')
        extra_kwargs = {
            'url': {'lookup_field': 'uuid', 'view_name': 'marketplace-category-detail'},
        }


PriceSerializer = serializers.DecimalField(
    min_value=0,
    max_digits=models.PlanComponent.PRICE_MAX_DIGITS,
    decimal_places=models.PlanComponent.PRICE_DECIMAL_PLACES,
)


class BasePlanSerializer(core_serializers.AugmentedSerializerMixin,
                         serializers.HyperlinkedModelSerializer):
    prices = serializers.DictField(child=PriceSerializer, write_only=True, required=False)
    quotas = serializers.DictField(child=serializers.IntegerField(min_value=0),
                                   write_only=True, required=False)

    class Meta(object):
        model = models.Plan
        fields = ('url', 'uuid', 'name', 'description', 'unit_price', 'unit',
                  'prices', 'quotas', 'max_amount', 'archived', 'is_active')
        read_ony_fields = ('unit_price', 'archived')
        extra_kwargs = {
            'url': {'lookup_field': 'uuid', 'view_name': 'marketplace-plan-detail'},
        }

    def get_fields(self):
        fields = super(BasePlanSerializer, self).get_fields()
        method = self.context['view'].request.method
        if method == 'GET':
            fields['prices'] = serializers.SerializerMethodField()
            fields['quotas'] = serializers.SerializerMethodField()
        return fields

    def get_prices(self, plan):
        return {item.component.type: item.price for item in plan.components.all()}

    def get_quotas(self, plan):
        return {item.component.type: item.amount for item in plan.components.all()}


class PlanDetailsSerializer(BasePlanSerializer):
    class Meta(BasePlanSerializer.Meta):
        model = models.Plan
        fields = BasePlanSerializer.Meta.fields + ('offering',)
        protected_fields = ('offering',)
        extra_kwargs = {
            'url': {'lookup_field': 'uuid', 'view_name': 'marketplace-plan-detail'},
            'offering': {'lookup_field': 'uuid', 'view_name': 'marketplace-offering-detail'},
        }

    def validate(self, attrs):
        if not self.instance:
            structure_permissions.is_owner(self.context['request'], None, attrs['offering'].customer)

        return attrs


class PlanUsageRequestSerializer(serializers.Serializer):
    offering_uuid = serializers.UUIDField(required=False)
    customer_provider_uuid = serializers.UUIDField(required=False)
    o = serializers.ChoiceField(choices=(
        'usage', 'limit', 'remaining',
        '-usage', '-limit', '-remaining',
    ), required=False)


class PlanUsageResponseSerializer(serializers.Serializer):
    plan_uuid = serializers.ReadOnlyField(source='uuid')
    plan_name = serializers.ReadOnlyField(source='name')

    limit = serializers.ReadOnlyField()
    usage = serializers.ReadOnlyField()
    remaining = serializers.ReadOnlyField()

    offering_uuid = serializers.ReadOnlyField(source='offering.uuid')
    offering_name = serializers.ReadOnlyField(source='offering.name')

    customer_provider_uuid = serializers.ReadOnlyField(source='offering.customer.uuid')
    customer_provider_name = serializers.ReadOnlyField(source='offering.customer.name')


class NestedScreenshotSerializer(serializers.ModelSerializer):
    class Meta(object):
        model = models.Screenshot
        fields = ('name', 'description', 'image', 'thumbnail')


class NestedOfferingFileSerializer(serializers.ModelSerializer):
    class Meta(object):
        model = models.OfferingFile
        fields = ('name', 'created', 'file',)


class ScreenshotSerializer(core_serializers.AugmentedSerializerMixin,
                           serializers.HyperlinkedModelSerializer):
    class Meta(object):
        model = models.Screenshot
        fields = ('url', 'uuid', 'name', 'description', 'image', 'thumbnail', 'offering')
        protected_fields = ('offering', 'image')
        extra_kwargs = {
            'url': {'lookup_field': 'uuid'},
            'offering': {'lookup_field': 'uuid', 'view_name': 'marketplace-offering-detail'},
        }

    def validate(self, attrs):
        if not self.instance:
            structure_permissions.is_owner(self.context['request'], None, attrs['offering'].customer)
        return attrs


FIELD_TYPES = (
    'boolean',
    'integer',
    'money',
    'string',
    'text',
    'html_text',
    'select_string',
    'select_openstack_tenant',
    'date',
    'time',
)


class DefaultField(serializers.Field):
    def to_internal_value(self, data):
        return data


class OptionFieldSerializer(serializers.Serializer):
    type = serializers.ChoiceField(choices=FIELD_TYPES)
    label = serializers.CharField()
    help_text = serializers.CharField(required=False)
    required = serializers.BooleanField(default=False)
    choices = serializers.ListField(child=serializers.CharField(), required=False)
    default = DefaultField(required=False)
    min = serializers.IntegerField(required=False)
    max = serializers.IntegerField(required=False)


class OfferingOptionsSerializer(serializers.Serializer):
    order = serializers.ListField(child=serializers.CharField())
    options = serializers.DictField(child=OptionFieldSerializer())


class OfferingComponentSerializer(serializers.ModelSerializer):
    class Meta(object):
        model = models.OfferingComponent
        fields = ('billing_type', 'type', 'name', 'description', 'measured_unit',
                  'limit_period', 'limit_amount', 'disable_quotas', 'product_code', 'article_code')
        extra_kwargs = {
            'billing_type': {'required': True},
        }


class OfferingDetailsSerializer(core_serializers.AugmentedSerializerMixin,
                                core_serializers.RestrictedSerializerMixin,
                                serializers.HyperlinkedModelSerializer):

    attributes = serializers.JSONField(required=False)
    options = serializers.JSONField(required=False)
    components = OfferingComponentSerializer(required=False, many=True)
    geolocations = core_serializers.GeoLocationField(required=False)
    order_item_count = serializers.SerializerMethodField()
    plans = BasePlanSerializer(many=True, required=False)
    screenshots = NestedScreenshotSerializer(many=True, read_only=True)
    state = serializers.ReadOnlyField(source='get_state_display')
    scope = GenericRelatedField(read_only=True)
    scope_uuid = serializers.ReadOnlyField(source='scope.uuid')
    files = NestedOfferingFileSerializer(many=True, read_only=True)
    quotas = serializers.SerializerMethodField()

    class Meta(object):
        model = models.Offering
        fields = ('url', 'uuid', 'created', 'name', 'description', 'full_description', 'terms_of_service',
                  'customer', 'customer_uuid', 'customer_name',
                  'category', 'category_uuid', 'category_title',
                  'rating', 'attributes', 'options', 'components', 'geolocations',
                  'state', 'native_name', 'native_description', 'vendor_details',
                  'thumbnail', 'order_item_count', 'plans', 'screenshots', 'type', 'shared', 'billable',
                  'scope', 'scope_uuid', 'files', 'quotas')
        related_paths = {
            'customer': ('uuid', 'name'),
            'category': ('uuid', 'title'),
        }
        protected_fields = ('customer', 'type')
        read_only_fields = ('state',)
        extra_kwargs = {
            'url': {'lookup_field': 'uuid', 'view_name': 'marketplace-offering-detail'},
            'customer': {'lookup_field': 'uuid', 'view_name': 'customer-detail'},
            'category': {'lookup_field': 'uuid', 'view_name': 'marketplace-category-detail'},
        }

    def get_order_item_count(self, offering):
        try:
            return offering.quotas.get(name='order_item_count').usage
        except ObjectDoesNotExist:
            return 0

    def get_quotas(self, offering):
        if offering.scope and hasattr(offering.scope, 'quotas'):
            return BasicQuotaSerializer(offering.scope.quotas, many=True, context=self.context).data


class OfferingModifySerializer(OfferingDetailsSerializer):

    def validate(self, attrs):
        if not self.instance:
            structure_permissions.is_owner(self.context['request'], None, attrs['customer'])

        self._validate_attributes(attrs)
        self._validate_plans(attrs)

        return attrs

    def validate_type(self, offering_type):
        if offering_type not in plugins.manager.backends.keys():
            raise rf_exceptions.ValidationError(_('Invalid value.'))
        return offering_type

    def _validate_attributes(self, attrs):
        category = attrs.get('category')
        if category is None and self.instance:
            category = self.instance.category

        attributes = attrs.get('attributes')
        if attributes is not None and not isinstance(attributes, dict):
            raise rf_exceptions.ValidationError({
                'attributes': _('Dictionary is expected.'),
            })

        if attributes is None and self.instance:
            attributes = self.instance.attributes

        category_attributes = models.Attribute.objects.filter(section__category=category)
        required_attributes = category_attributes.filter(required=True).values_list('key', flat=True)
        missing_attributes = set(required_attributes) - (set(attributes.keys()) if attributes else set())

        if missing_attributes:
            raise rf_exceptions.ValidationError({
                'attributes': _('These attributes are required: %s' % ', '.join(sorted(missing_attributes)))
            })

        for attribute in category_attributes:
            value = attributes.get(attribute.key)
            if value is None:
                # Use default attribute value if it is defined
                if attribute.default is not None:
                    attributes[attribute.key] = attribute.default
                continue

            validator = attribute_types.get_attribute_type(attribute.type)
            if not validator:
                continue

            try:
                validator.validate(value, list(attribute.options.values_list('key', flat=True)))
            except ValidationError as e:
                raise rf_exceptions.ValidationError({attribute.key: e.message})

    def validate_options(self, options):
        serializer = OfferingOptionsSerializer(data=options)
        serializer.is_valid(raise_exception=True)
        return serializer.validated_data

    def _validate_plans(self, attrs):
        custom_components = attrs.get('components')
        if not custom_components and self.instance:
            custom_components = self.instance.components.all().values()

        offering_type = attrs.get('type', getattr(self.instance, 'type', None))
        builtin_components = plugins.manager.get_components(offering_type)

        valid_types = set()
        fixed_types = set()

        if builtin_components and attrs.get('components'):
            raise serializers.ValidationError({
                'components': _('Extra components are not allowed.')
            })

        elif builtin_components:
            valid_types = {component.type for component in builtin_components}
            fixed_types = {component.type
                           for component in plugins.manager.get_components(offering_type)
                           if component.billing_type == models.OfferingComponent.BillingTypes.FIXED}

        elif custom_components:
            valid_types = {component['type'] for component in custom_components}
            fixed_types = {component['type'] for component in custom_components
                           if component['billing_type'] == models.OfferingComponent.BillingTypes.FIXED}

        for plan in attrs.get('plans', []):
            plan_name = plan.get('name')

            prices = plan.get('prices', {})
            invalid_components = ', '.join(sorted(set(prices.keys()) - valid_types))
            if invalid_components:
                raise serializers.ValidationError({
                    'plans': _('Invalid price components %s in plan "%s".') % (invalid_components, plan_name)
                })

            quotas = plan.get('quotas', {})
            invalid_components = ', '.join(sorted(set(quotas.keys()) - fixed_types))
            if invalid_components:
                raise serializers.ValidationError({
                    'plans': _('Invalid quota components %s in plan "%s".') % (invalid_components, plan_name),
                })

            plan['unit_price'] = sum(
                prices.get(component, 0) * quotas.get(component, 0)
                for component in fixed_types
            )

    def _create_plan(self, offering, plan_data, components):
        quotas = plan_data.pop('quotas', {})
        prices = plan_data.pop('prices', {})
        plan = models.Plan.objects.create(offering=offering, **plan_data)

        for name, component in components.items():
            models.PlanComponent.objects.create(
                plan=plan,
                component=component,
                amount=quotas.get(name) or 0,
                price=prices.get(name) or 0,
            )

    def _create_components(self, offering, custom_components):
        fixed_components = plugins.manager.get_components(offering.type)

        for component_data in fixed_components:
            models.OfferingComponent.objects.create(
                offering=offering,
                **component_data._asdict()
            )

        for component_data in custom_components:
            models.OfferingComponent.objects.create(offering=offering, **component_data)

    def _create_plans(self, offering, plans):
        components = {component.type: component for component in offering.components.all()}
        for plan_data in plans:
            self._create_plan(offering, plan_data, components)


class OfferingCreateSerializer(OfferingModifySerializer):
    class Meta(OfferingModifySerializer.Meta):
        fields = OfferingModifySerializer.Meta.fields + ('service_attributes',)

    service_attributes = serializers.JSONField(required=False, write_only=True)

    def validate_plans(self, plans):
        if len(plans) < 1:
            raise serializers.ValidationError({
                'plans': _('At least one plan should be specified.')
            })
        return plans

    @transaction.atomic
    def create(self, validated_data):
        plans = validated_data.pop('plans', [])
        custom_components = validated_data.pop('components', [])

        offering_type = validated_data.get('type')
        service_type = plugins.manager.get_service_type(offering_type)
        if service_type:
            validated_data = self._create_service(service_type, validated_data)

        offering = super(OfferingCreateSerializer, self).create(validated_data)
        self._create_components(offering, custom_components)
        self._create_plans(offering, plans)

        return offering

    def _create_service(self, service_type, validated_data):
        """
        Marketplace offering model does not accept service_attributes field as is,
        therefore we should remove it from validated_data and create service settings object.
        Then we need to specify created object and offering's scope.
        """
        name = validated_data['name']
        service_attributes = validated_data.pop('service_attributes', {})
        if not service_attributes:
            raise ValidationError({
                'service_attributes': _('This field is required.')
            })
        payload = dict(
            name=name,
            # It is expected that customer URL is passed to the service settings serializer
            customer=self.initial_data['customer'],
            type=service_type,
            **service_attributes
        )
        serializer_class = SupportedServices.get_service_serializer_for_key(service_type)
        serializer = serializer_class(data=payload, context=self.context)
        serializer.is_valid(raise_exception=True)
        service = serializer.save()
        # Usually we don't allow users to create new shared service settings via REST API.
        # That's shared flag is marked as read-only in service settings serializer.
        # But shared offering should be created with shared service settings.
        # That's why we set it to shared only after service settings object is created.
        if validated_data.get('shared'):
            service.settings.shared = True
            service.settings.save()
            # Usually connect shared settings task is called when service is created.
            # But as we set shared flag after serializer has been executed,
            # we need to connect shared settings manually.
            connect_shared_settings(service.settings)
        validated_data['scope'] = service.settings
        return validated_data


class PlanUpdateSerializer(BasePlanSerializer):

    class Meta(BasePlanSerializer.Meta):
        extra_kwargs = {
            'uuid': {'read_only': False},
        }


class OfferingUpdateSerializer(OfferingModifySerializer):

    plans = PlanUpdateSerializer(many=True, required=False, write_only=True)

    def _update_components(self, instance, components):
        resources_exist = models.Resource.objects.filter(offering=instance).exists()

        old_components = {
            component.type: component
            for component in instance.components.all()
        }

        new_components = {
            component['type']: models.OfferingComponent(offering=instance, **component)
            for component in components
        }

        removed_components = set(old_components.keys()) - set(new_components.keys())
        added_components = set(new_components.keys()) - set(old_components.keys())
        updated_components = set(new_components.keys()) & set(old_components.keys())

        if removed_components:
            if resources_exist:
                raise serializers.ValidationError({
                    'components': _('These components cannot be removed because they are already used: %s') %
                    ', '.join(removed_components)
                })
            else:
                models.OfferingComponent.objects.filter(type__in=removed_components).delete()

        for key in added_components:
            new_components[key].save()

        COMPONENT_KEYS = (
            'name', 'description',
            'billing_type', 'measured_unit',
            'limit_period', 'limit_amount', 'disable_quotas',
            'product_code', 'article_code',
        )

        for component_key in updated_components:
            new_component = new_components[component_key]
            old_component = old_components[component_key]
            for key in COMPONENT_KEYS:
                setattr(old_component, key, getattr(new_component, key))
            old_component.save()

    def _update_plan_components(self, old_plan, new_plan):
        new_quotas = new_plan.get('quotas', {})
        new_prices = new_plan.get('prices', {})

        new_keys = set(new_quotas.keys()) | set(new_prices.keys())
        old_keys = set(old_plan.components.values_list('component__type', flat=True))

        for key in new_keys - old_keys:
            component = old_plan.offering.components.get(type=key)
            models.PlanComponent.objects.create(plan=old_plan, component=component)

    def _update_quotas(self, old_plan, new_plan):
        new_quotas = new_plan.get('quotas', {})
        new_prices = new_plan.get('prices', {})
        component_map = {
            component.component.type: component
            for component in old_plan.components.all()
        }
        for key, old_component in component_map.items():
            new_amount = new_quotas.get(key, 0)
            if old_component.amount != new_amount:
                old_component.amount = new_amount
                old_component.save(update_fields=['amount'])

            new_price = new_prices.get(key, 0)
            if old_component.price != new_price:
                old_component.price = new_price
                old_component.save(update_fields=['price'])

    def _update_plan_details(self, old_plan, new_plan):
        PLAN_FIELDS = (
            'name', 'description',
            'unit', 'max_amount',
            'product_code', 'article_code',
        )

        for key in PLAN_FIELDS:
            if key in new_plan:
                setattr(old_plan, key, new_plan.get(key))
        old_plan.save()

    def _update_plans(self, offering, new_plans):
        old_plans = offering.plans.all()
        old_ids = set(old_plans.values_list('uuid', flat=True))

        new_map = {plan['uuid']: plan for plan in new_plans if 'uuid' in plan}
        added_plans = [plan for plan in new_plans if 'uuid' not in plan]

        removed_ids = set(old_ids) - set(new_map.keys())
        updated_ids = set(new_map.keys()) & set(old_ids)

        removed_plans = models.Plan.objects.filter(uuid__in=removed_ids).exclude(archived=True)
        updated_plans = {plan.uuid: plan for plan in models.Plan.objects.filter(uuid__in=updated_ids)}

        for plan_uuid, old_plan in updated_plans.items():
            new_plan = new_map[plan_uuid]
            self._update_plan_details(old_plan, new_plan)
            self._update_plan_components(old_plan, new_plan)
            self._update_quotas(old_plan, new_plan)

        if added_plans:
            self._create_plans(offering, added_plans)

        for plan in removed_plans:
            plan.archived = True
            plan.save()

    @transaction.atomic
    def update(self, instance, validated_data):
        if 'components' in validated_data:
            components = validated_data.pop('components', [])
            self._update_components(instance, components)
        if 'plans' in validated_data:
            new_plans = validated_data.pop('plans', [])
            self._update_plans(instance, new_plans)
        offering = super(OfferingUpdateSerializer, self).update(instance, validated_data)
        return offering


class ComponentQuotaSerializer(serializers.ModelSerializer):
    type = serializers.ReadOnlyField(source='component.type')

    class Meta(object):
        model = models.ComponentQuota
        fields = ('type', 'limit', 'usage')


class BaseItemSerializer(core_serializers.AugmentedSerializerMixin,
                         serializers.HyperlinkedModelSerializer,
                         core_serializers.RestrictedSerializerMixin):

    class Meta(object):
        fields = ('offering', 'offering_name', 'offering_uuid',
                  'offering_description', 'offering_thumbnail', 'offering_type',
                  'offering_terms_of_service', 'offering_shared', 'offering_billable',
                  'provider_name', 'provider_uuid',
                  'category_title', 'category_uuid',
                  'plan', 'plan_unit', 'plan_name', 'plan_uuid', 'plan_description',
                  'attributes', 'limits', 'uuid', 'created')
        related_paths = {
            'offering': ('name', 'uuid', 'description', 'type', 'terms_of_service', 'shared', 'billable'),
            'plan': ('unit', 'uuid', 'name', 'description')
        }
        protected_fields = ('offering',)
        extra_kwargs = {
            'offering': {'lookup_field': 'uuid', 'view_name': 'marketplace-offering-detail'},
            'plan': {'lookup_field': 'uuid', 'view_name': 'marketplace-plan-detail'},
        }

    provider_name = serializers.ReadOnlyField(source='offering.customer.name')
    provider_uuid = serializers.ReadOnlyField(source='offering.customer.uuid')
    category_title = serializers.ReadOnlyField(source='offering.category.title')
    category_uuid = serializers.ReadOnlyField(source='offering.category.uuid')
    offering_thumbnail = serializers.FileField(source='offering.thumbnail', read_only=True)

    def validate_offering(self, offering):
        if not offering.state == models.Offering.States.ACTIVE:
            raise rf_exceptions.ValidationError(_('Offering is not available.'))
        return offering

    def validate(self, attrs):
        offering = attrs.get('offering')
        plan = attrs.get('plan')

        if not offering:
            if not self.instance:
                raise rf_exceptions.ValidationError({
                    'offering': _('This field is required.')
                })
            offering = self.instance.offering

        if plan:
            if plan.offering != offering:
                raise rf_exceptions.ValidationError({
                    'plan': _('This plan is not available for selected offering.')
                })

            validate_plan(plan)

        if offering.options:
            validate_options(offering.options['options'], attrs.get('attributes'))

        limits = attrs.get('limits')
        if limits:
            valid_component_types = offering.components \
                .filter(billing_type=models.OfferingComponent.BillingTypes.USAGE) \
                .exclude(disable_quotas=True) \
                .values_list('type', flat=True)
            invalid_types = set(limits.keys()) - set(valid_component_types)
            if invalid_types:
                raise ValidationError({'limits': _('Invalid types: %s') % ', '.join(invalid_types)})
        return attrs


class BaseRequestSerializer(BaseItemSerializer):
    type = NaturalChoiceField(
        choices=models.RequestTypeMixin.Types.CHOICES,
        required=False,
        default=models.RequestTypeMixin.Types.CREATE,
    )

    class Meta(BaseItemSerializer.Meta):
        fields = BaseItemSerializer.Meta.fields + ('type',)


class NestedOrderItemSerializer(BaseRequestSerializer):
    class Meta(BaseRequestSerializer.Meta):
        model = models.OrderItem
        fields = BaseRequestSerializer.Meta.fields + (
            'resource_uuid', 'resource_type', 'resource_name',
            'cost', 'state', 'marketplace_resource_uuid', 'error_message',
        )

        read_only_fields = ('cost', 'state', 'error_message')
        protected_fields = ('offering', 'plan')

    marketplace_resource_uuid = serializers.ReadOnlyField(source='resource.uuid')
    resource_name = serializers.ReadOnlyField(source='resource.name')
    resource_uuid = serializers.ReadOnlyField(source='resource.backend_uuid')
    resource_type = serializers.ReadOnlyField(source='resource.backend_type')
    state = serializers.ReadOnlyField(source='get_state_display')
    limits = serializers.DictField(child=serializers.IntegerField(), required=False)

    def get_fields(self):
        fields = super(BaseItemSerializer, self).get_fields()
        method = self.context['view'].request.method
        if method == 'GET':
            fields['attributes'] = serializers.ReadOnlyField(source='safe_attributes')
        return fields


class OrderItemDetailsSerializer(NestedOrderItemSerializer):
    class Meta(NestedOrderItemSerializer.Meta):
        fields = NestedOrderItemSerializer.Meta.fields + (
            'order_uuid', 'order_approved_at', 'order_approved_by',
            'created_by_full_name', 'created_by_civil_number',
            'customer_name', 'customer_uuid',
            'project_name', 'project_uuid',
            'old_plan_name', 'new_plan_name',
            'old_plan_uuid', 'new_plan_uuid',
            'old_cost_estimate', 'new_cost_estimate',
            'can_terminate',
        )

    order_uuid = serializers.ReadOnlyField(source='order.uuid')
    order_approved_at = serializers.ReadOnlyField(source='order.approved_at')
    order_approved_by = serializers.ReadOnlyField(source='order.approved_by.full_name')

    created_by_full_name = serializers.ReadOnlyField(source='order.created_by.full_name')
    created_by_civil_number = serializers.ReadOnlyField(source='order.created_by.civil_number')

    customer_name = serializers.ReadOnlyField(source='order.project.customer.name')
    customer_uuid = serializers.ReadOnlyField(source='order.project.customer.uuid')

    project_name = serializers.ReadOnlyField(source='order.project.name')
    project_uuid = serializers.ReadOnlyField(source='order.project.uuid')

    old_plan_name = serializers.ReadOnlyField(source='old_plan.name')
    new_plan_name = serializers.ReadOnlyField(source='plan.name')

    old_plan_uuid = serializers.ReadOnlyField(source='old_plan.uuid')
    new_plan_uuid = serializers.ReadOnlyField(source='plan.uuid')

    old_cost_estimate = serializers.ReadOnlyField(source='resource.cost')
    new_cost_estimate = serializers.ReadOnlyField(source='cost')

    can_terminate = serializers.SerializerMethodField()

    def get_can_terminate(self, order_item):
        if not plugins.manager.can_terminate_order_item(order_item.offering.type):
            return False

        if order_item.state not in (models.OrderItem.States.PENDING, models.OrderItem.States.EXECUTING):
            return False

        return True


class CartItemSerializer(BaseRequestSerializer):
    limits = serializers.DictField(child=serializers.IntegerField(), required=False)
    estimate = serializers.ReadOnlyField(source='cost')

    class Meta(BaseRequestSerializer.Meta):
        model = models.CartItem
        fields = BaseRequestSerializer.Meta.fields + ('estimate',)

    @transaction.atomic
    def create(self, validated_data):
        validated_data['user'] = self.context['request'].user
        item = super(CartItemSerializer, self).create(validated_data)
        item.init_cost()
        item.save(update_fields=['cost'])
        return item


class CartSubmitSerializer(serializers.Serializer):
    project = serializers.HyperlinkedRelatedField(
        queryset=structure_models.Project.objects.all(),
        view_name='project-detail',
        lookup_field='uuid',
        required=True,
    )

    def get_fields(self):
        fields = super(CartSubmitSerializer, self).get_fields()
        project_field = fields['project']
        project_field.queryset = filter_queryset_for_user(
            project_field.queryset, self.context['request'].user)
        return fields

    @transaction.atomic()
    def create(self, validated_data):
        user = self.context['request'].user

        items = models.CartItem.objects.filter(user=user)
        if items.count() == 0:
            raise serializers.ValidationError(_('Shopping cart is empty'))

        project = validated_data['project']
        order = create_order(project, user, items, self.context['request'])
        items.delete()
        return order


def check_availability_of_auto_approving(items, user, project):
    if user.is_staff:
        return True

    # Skip approval of private offering for project users
    if all(item.offering.is_private for item in items):
        return structure_permissions._has_admin_access(user, project)

    return permissions.user_can_approve_order(user, project)


def create_order(project, user, items, request):
    order_params = dict(project=project, created_by=user)
    order = models.Order.objects.create(**order_params)

    for item in items:
        if item.type in (models.OrderItem.Types.UPDATE, models.OrderItem.Types.TERMINATE) and \
                item.resource and models.OrderItem.objects.filter(
            resource=item.resource,
            state__in=(models.OrderItem.States.PENDING, models.OrderItem.States.EXECUTING)
        ).exists():
            raise rf_exceptions.ValidationError(_('Pending order item for resource already exists.'))

        try:
            order_item = order.add_item(
                offering=item.offering,
                attributes=item.attributes,
                resource=getattr(item, 'resource', None),  # cart item does not have resource
                plan=item.plan,
                old_plan=getattr(item, 'old_plan', None),  # cart item does not have old plan
                limits=item.limits,
                type=item.type,
            )
        except ValidationError as e:
            raise rf_exceptions.ValidationError(e)
        validate_order_item(order_item, request)

    order.init_total_cost()
    order.save()

    if check_availability_of_auto_approving(items, user, project):
        tasks.approve_order(order, user)
    else:
        transaction.on_commit(lambda: tasks.notify_order_approvers.delay(order.uuid))

    return order


class OrderSerializer(structure_serializers.PermissionFieldFilteringMixin,
                      core_serializers.AugmentedSerializerMixin,
                      serializers.HyperlinkedModelSerializer):

    state = serializers.ReadOnlyField(source='get_state_display')
    items = NestedOrderItemSerializer(many=True)

    class Meta(object):
        model = models.Order
        fields = ('url', 'uuid',
                  'created', 'created_by', 'created_by_username', 'created_by_full_name',
                  'approved_by', 'approved_at', 'approved_by_username', 'approved_by_full_name',
                  'project', 'state', 'items', 'total_cost', 'file')
        read_only_fields = ('created_by', 'approved_by', 'approved_at', 'state', 'total_cost')
        protected_fields = ('project', 'items')
        related_paths = {
            'created_by': ('username', 'full_name'),
            'approved_by': ('username', 'full_name'),
        }
        extra_kwargs = {
            'url': {'lookup_field': 'uuid'},
            'created_by': {'lookup_field': 'uuid', 'view_name': 'user-detail'},
            'approved_by': {'lookup_field': 'uuid', 'view_name': 'user-detail'},
            'project': {'lookup_field': 'uuid', 'view_name': 'project-detail'},
        }

    file = serializers.SerializerMethodField()

    def get_file(self, obj):
        if not obj.has_file():
            return None

        return reverse('marketplace-order-pdf',
                       kwargs={'uuid': obj.uuid},
                       request=self.context['request'])

    @transaction.atomic
    def create(self, validated_data):
        request = self.context['request']
        project = validated_data['project']
        items = [
            models.OrderItem(
                offering=item['offering'],
                plan=item.get('plan'),
                attributes=item.get('attributes', {}),
                limits=item.get('limits', {}),
                type=item.get('type'),
            )
            for item in validated_data['items']
        ]
        return create_order(project, request.user, items, request)

    def get_filtered_field_names(self):
        return 'project',


class CustomerOfferingSerializer(serializers.HyperlinkedModelSerializer):
    offering_set = serializers.HyperlinkedRelatedField(
        many=True,
        view_name='marketplace-offering-detail',
        lookup_field='uuid',
        queryset=models.Offering.objects.all()
    )

    class Meta(object):
        model = structure_models.Customer
        fields = ('offering_set',)


class ResourceSerializer(BaseItemSerializer):
    class Meta(BaseItemSerializer.Meta):
        model = models.Resource
        fields = BaseItemSerializer.Meta.fields + (
            'scope', 'state', 'resource_uuid', 'resource_type',
            'project', 'project_uuid', 'project_name',
            'customer_uuid', 'customer_name',
            'offering_uuid', 'offering_name',
            'backend_metadata', 'is_usage_based', 'name',
        )
        read_only_fields = ('backend_metadata', 'scope',)

    state = serializers.ReadOnlyField(source='get_state_display')
    scope = core_serializers.GenericRelatedField()
    resource_uuid = serializers.ReadOnlyField(source='backend_uuid')
    resource_type = serializers.ReadOnlyField(source='backend_type')
    project = serializers.HyperlinkedRelatedField(
        lookup_field='uuid',
        view_name='project-detail',
        read_only=True,
    )
    project_uuid = serializers.ReadOnlyField(source='project.uuid')
    project_name = serializers.ReadOnlyField(source='project.name')
    customer_uuid = serializers.ReadOnlyField(source='project.customer.uuid')
    customer_name = serializers.ReadOnlyField(source='project.customer.name')
    offering_uuid = serializers.ReadOnlyField(source='offering.uuid')
    offering_name = serializers.ReadOnlyField(source='offering.name')
    # If resource is usage-based, frontend would render button to show and report usage
    is_usage_based = serializers.ReadOnlyField(source='offering.is_usage_based')


class ResourceSwitchPlanSerializer(serializers.HyperlinkedModelSerializer):
    class Meta(object):
        model = models.Resource
        fields = ('plan',)

    plan = serializers.HyperlinkedRelatedField(
        view_name='marketplace-plan-detail',
        lookup_field='uuid',
        queryset=models.Plan.objects.all(),
        required=True,
    )

    def validate(self, attrs):
        plan = attrs['plan']
        resource = self.context['view'].get_object()

        if plan.offering != resource.offering:
            raise rf_exceptions.ValidationError({
                'plan': _('Plan is not available for this offering.')
            })

        validate_plan(plan)
        return attrs


class BaseComponentSerializer(serializers.Serializer):
    type = serializers.ReadOnlyField(source='component.type')
    name = serializers.ReadOnlyField(source='component.name')
    measured_unit = serializers.ReadOnlyField(source='component.measured_unit')


class CategoryComponentUsageSerializer(core_serializers.RestrictedSerializerMixin,
                                       BaseComponentSerializer,
                                       serializers.ModelSerializer):
    category_title = serializers.ReadOnlyField(source='component.category.title')
    category_uuid = serializers.ReadOnlyField(source='component.category.uuid')
    scope = GenericRelatedField(related_models=(structure_models.Project, structure_models.Customer))

    class Meta(object):
        model = models.CategoryComponentUsage
        fields = ('name', 'type', 'measured_unit', 'category_title', 'category_uuid',
                  'date', 'reported_usage', 'fixed_usage', 'scope')


class BaseComponentUsageSerializer(BaseComponentSerializer, serializers.ModelSerializer):
    class Meta(object):
        model = models.ComponentUsage
        fields = (
            'uuid', 'created', 'description',
            'type', 'name', 'measured_unit', 'usage', 'date',
        )


class ComponentUsageSerializer(BaseComponentUsageSerializer):
    resource_name = serializers.ReadOnlyField(source='resource.name')
    resource_uuid = serializers.ReadOnlyField(source='resource.uuid')

    offering_name = serializers.ReadOnlyField(source='resource.offering.name')
    offering_uuid = serializers.ReadOnlyField(source='resource.offering.uuid')

    project_name = serializers.ReadOnlyField(source='resource.project.name')
    project_uuid = serializers.ReadOnlyField(source='resource.project.uuid')

    customer_name = serializers.ReadOnlyField(source='resource.project.customer.name')
    customer_uuid = serializers.ReadOnlyField(source='resource.project.customer.uuid')

    class Meta(BaseComponentUsageSerializer.Meta):
        fields = BaseComponentUsageSerializer.Meta.fields + (
            'resource_name', 'resource_uuid',
            'offering_name', 'offering_uuid',
            'project_name', 'project_uuid',
            'customer_name', 'customer_uuid',
        )


class ResourcePlanPeriodSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.ResourcePlanPeriod
        fields = ('uuid', 'plan_name', 'plan_uuid', 'start', 'end', 'components')

    plan_name = serializers.ReadOnlyField(source='plan.name')
    plan_uuid = serializers.ReadOnlyField(source='plan.uuid')
    components = BaseComponentUsageSerializer(many=True)


class ServiceProviderSignatureSerializer(serializers.Serializer):
    customer = serializers.SlugRelatedField(queryset=structure_models.Customer.objects.all(), slug_field='uuid')
    data = serializers.CharField()
    dry_run = serializers.BooleanField(default=False, required=False)

    def validate(self, attrs):
        customer = attrs['customer']
        service_provider = getattr(customer, 'serviceprovider', None)
        api_secret_code = service_provider and service_provider.api_secret_code

        if not api_secret_code:
            raise rf_exceptions.ValidationError(_('API secret code is not set.'))

        try:
            data = utils.decode_api_data(attrs['data'], api_secret_code)
            attrs['data'] = data
            return attrs
        except jwt.exceptions.DecodeError:
            raise rf_exceptions.ValidationError(_('Signature verification failed.'))


class ComponentUsageItemSerializer(serializers.Serializer):
    type = serializers.CharField()
    amount = serializers.IntegerField()
    description = serializers.CharField(required=False, allow_blank=True)


class ComponentUsageCreateSerializer(serializers.Serializer):
    usages = ComponentUsageItemSerializer(many=True)
    plan_period = serializers.SlugRelatedField(queryset=models.ResourcePlanPeriod.objects.all(), slug_field='uuid')

    def validate_plan_period(self, plan_period):
        date = datetime.date.today()
        if plan_period.end and plan_period.end < core_utils.month_start(date):
            raise serializers.ValidationError(_('Billing period is closed.'))
        return plan_period

    def validate(self, attrs):
        attrs = super(ComponentUsageCreateSerializer, self).validate(attrs)
        plan_period = attrs['plan_period']
        resource = plan_period.resource
        offering = resource.plan.offering

        if resource.state == models.Resource.States.TERMINATED:
            raise rf_exceptions.ValidationError({
                'resource': _('Resource is terminated.')
            })

        valid_components = set(offering.get_usage_components().keys())
        actual_components = {usage['type'] for usage in attrs['usages']}
        invalid_components = ', '.join(sorted(valid_components - actual_components))

        if invalid_components:
            raise rf_exceptions.ValidationError(_('These components are invalid: %s.') % invalid_components)

        return attrs

    def save(self):
        plan_period = self.validated_data['plan_period']
        resource = plan_period.resource
        components = resource.plan.offering.get_usage_components()
        date = datetime.date.today()

        for usage in self.validated_data['usages']:
            amount = usage['amount']
            description = usage.get('description', '')
            component = components[usage['type']]
            component.validate_amount(resource, amount, date)

            models.ComponentUsage.objects.update_or_create(
                resource=resource,
                component=component,
                plan_period=plan_period,
                defaults={'usage': amount, 'date': date, 'description': description},
            )


class OfferingFileSerializer(core_serializers.AugmentedSerializerMixin,
                             serializers.HyperlinkedModelSerializer):
    class Meta(object):
        model = models.OfferingFile
        fields = ('url', 'uuid', 'name', 'offering', 'created', 'file',)
        extra_kwargs = dict(
            url={'lookup_field': 'uuid'},
            offering={'lookup_field': 'uuid', 'view_name': 'marketplace-offering-detail'},
        )


def validate_plan(plan):
    """"
    Ensure that maximum amount of resources with current plan is not reached yet.
    """
    if not plan.is_active:
        raise rf_exceptions.ValidationError({
            'plan': _('Plan is not available because limit has been reached.')
        })


def get_is_service_provider(serializer, scope):
    customer = structure_permissions._get_customer(scope)
    return models.ServiceProvider.objects.filter(customer=customer).exists()


def add_service_provider(sender, fields, **kwargs):
    fields['is_service_provider'] = serializers.SerializerMethodField()
    setattr(sender, 'get_is_service_provider', get_is_service_provider)


def get_marketplace_offering_uuid(serializer, scope):
    try:
        return models.Resource.objects.get(scope=scope).offering.uuid
    except ObjectDoesNotExist:
        return


def get_marketplace_offering_name(serializer, scope):
    try:
        return models.Resource.objects.get(scope=scope).offering.name
    except ObjectDoesNotExist:
        return


def get_marketplace_category_uuid(serializer, scope):
    try:
        return models.Resource.objects.get(scope=scope).offering.category.uuid
    except ObjectDoesNotExist:
        return


def get_marketplace_category_name(serializer, scope):
    try:
        return models.Resource.objects.get(scope=scope).offering.category.title
    except ObjectDoesNotExist:
        return


def get_marketplace_resource_uuid(serializer, scope):
    try:
        return models.Resource.objects.get(scope=scope).uuid
    except ObjectDoesNotExist:
        return


def get_is_usage_based(serializer, scope):
    try:
        return models.Resource.objects.get(scope=scope).offering.is_usage_based
    except ObjectDoesNotExist:
        return


def add_marketplace_offering(sender, fields, **kwargs):
    fields['marketplace_offering_uuid'] = serializers.SerializerMethodField()
    setattr(sender, 'get_marketplace_offering_uuid', get_marketplace_offering_uuid)

    fields['marketplace_offering_name'] = serializers.SerializerMethodField()
    setattr(sender, 'get_marketplace_offering_name', get_marketplace_offering_name)

    fields['marketplace_category_uuid'] = serializers.SerializerMethodField()
    setattr(sender, 'get_marketplace_category_uuid', get_marketplace_category_uuid)

    fields['marketplace_category_name'] = serializers.SerializerMethodField()
    setattr(sender, 'get_marketplace_category_name', get_marketplace_category_name)

    fields['marketplace_resource_uuid'] = serializers.SerializerMethodField()
    setattr(sender, 'get_marketplace_resource_uuid', get_marketplace_resource_uuid)

    fields['is_usage_based'] = serializers.SerializerMethodField()
    setattr(sender, 'get_is_usage_based', get_is_usage_based)


core_signals.pre_serializer_fields.connect(
    sender=structure_serializers.CustomerSerializer,
    receiver=add_service_provider,
)

core_signals.pre_serializer_fields.connect(
    sender=support_serializers.OfferingSerializer,
    receiver=add_marketplace_offering,
)

for resource_serializer in SupportedServices.get_resource_serializers():
    core_signals.pre_serializer_fields.connect(
        sender=resource_serializer,
        receiver=add_marketplace_offering,
    )



""" Formatters, handlers and other stuff for default logging configuration """

import datetime
import json
import logging

from celery import current_app


class EventFormatter(logging.Formatter):

    def format_timestamp(self, time):
        return datetime.datetime.utcfromtimestamp(time).isoformat() + 'Z'

    def levelname_to_importance(self, levelname):
        if levelname == 'DEBUG':
            return 'low'
        elif levelname == 'INFO':
            return 'normal'
        elif levelname == 'WARNING':
            return 'high'
        elif levelname == 'ERROR':
            return 'very high'
        else:
            return 'critical'

    def format(self, record):
        message = {
            # basic
            '@timestamp': self.format_timestamp(record.created),
            '@version': 1,
            'message': record.getMessage(),

            # logging details
            'levelname': record.levelname,
            'logger': record.name,
            'importance': self.levelname_to_importance(record.levelname),
            'importance_code': record.levelno,
        }

        if hasattr(record, 'event_type'):
            message['event_type'] = record.event_type

        if hasattr(record, 'event_context'):
            message.update(record.event_context)

        return json.dumps(message)


class EventLoggerAdapter(logging.LoggerAdapter, object):
    """ LoggerAdapter """

    def __init__(self, logger):
        super(EventLoggerAdapter, self).__init__(logger, {})

    def process(self, msg, kwargs):
        if 'extra' in kwargs:
            kwargs['extra']['event'] = True
        else:
            kwargs['extra'] = {'event': True}
        return msg, kwargs


class RequireEvent(logging.Filter):
    """ A filter that allows only event records. """

    def filter(self, record):
        return getattr(record, 'event', False)


class RequireNotEvent(logging.Filter):
    """ A filter that allows only non-event records. """

    def filter(self, record):
        return not getattr(record, 'event', False)


class RequireNotBackgroundTask(logging.Filter):
    """ Filter out messages from background tasks """

    def filter(self, record):
        try:
            name = getattr(record, 'data', {})['name']
            task = current_app.tasks[name]
        except KeyError:
            return True
        is_background = getattr(task, 'is_background', False)
        return not is_background


class TCPEventHandler(logging.handlers.SocketHandler, object):

    def __init__(self, host='localhost', port=5959):
        super(TCPEventHandler, self).__init__(host, int(port))
        self.formatter = EventFormatter()

    def makePickle(self, record):
        return self.formatter.format(record) + b'\n'



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('openstack_tenant', '0013_init_backupschedule_spl'),
    ]

    operations = [
        migrations.AlterField(
            model_name='backupschedule',
            name='service_project_link',
            field=models.ForeignKey(related_name='backup_schedules', on_delete=django.db.models.deletion.PROTECT, to='openstack_tenant.OpenStackTenantServiceProjectLink'),
            preserve_default=False,
        ),
    ]



#!/usr/bin/env python
from setuptools import setup, find_packages

# apache-libcloud is required by AWS plugin
# defusedxml is required by djangosaml2
# jira is required by JIRA plugin
# lxml is required by waldur-auth-valimo
# passlib is required by Ansible plugin
# paypalrestsdk is required by PayPal plugin
# python-digitalocean is required by DigitalOcean plugin
# python-freeipa is required by FreeIPA plugin
install_requires = [
    'ansible-waldur-module>=0.8.2',
    'apache-libcloud>=1.1.0,<2.3.0',
    'azure-mgmt-compute==4.0.1',
    'azure-mgmt-network==2.0.0',
    'azure-mgmt-rdbms==1.2.0',
    'azure-mgmt-resource==2.0.0',
    'azure-mgmt-storage==2.0.0',
    'Babel!=2.4.0,>=2.3.4',
    'Celery>=4.2.0,<4.3.0',
    'cmd2<0.9.0',  # TODO: Drop restriction after Waldur is migrated to Python 3.
    'croniter>=0.3.4,<0.3.6',
    'defusedxml>=0.4.1',
    'django-admin-tools==0.8.0',
    'django-auth-ldap>=1.3.0',
    'django-defender>=0.5.3',
    'django-filter==1.0.2',
    'django-fluent-dashboard==0.6.1',
    'django-fsm==2.3.0',
    'django-jsoneditor>=0.0.7',
    'django-model-utils==3.0.0',
    'django-openid-auth>=0.14',
    'django-redis-cache>=1.6.5',
    'django-rest-swagger==2.1.2',
    'django-reversion==2.0.8',
    'django-taggit>=0.20.2',
    'Django>=1.11.20,<2.0',
    'djangorestframework>=3.6.3,<3.7.0',
    'djangosaml2==0.17.1',
    'hiredis>=0.2.0',
    'influxdb>=4.1.0',
    'iptools>=0.6.1',
    'jira>=1.0.15',
    'lxml>=3.2.0',
    'passlib>=1.7.0',
    'paypalrestsdk>=1.10.0,<2.0',
    'pbr!=2.1.0',
    'pdfkit>=0.6.1',
    'Pillow>=2.0.0',
    'PrettyTable<0.8,>=0.7.1',
    'psycopg2>=2.5.4',  # https://docs.djangoproject.com/en/1.11/ref/databases/#postgresql-notes
    'pycountry>=1.20,<2.0',
    'python-ceilometerclient>=2.9.0',
    'python-cinderclient>=3.1.0',
    'python-digitalocean>=1.5',
    'python-freeipa>=0.2.2',
    'python-glanceclient>=2.8.0',
    'python-keystoneclient>=3.13.0',
    'python-neutronclient>=6.5.0',
    'python-novaclient>=9.1.0',
    'pyvat>=1.3.1,<2.0',
    'PyYAML>=3.10',
    'pyzabbix>=0.7.2',
    'redis==2.10.6',
    'requests>=2.14.2',
    'sqlparse>=0.1.11',
    'pyjwt>=1.5.3',
]

test_requires = [
    'ddt>=1.0.0,<1.1.0',
    'docker',
    'factory_boy==2.4.1',
    'freezegun==0.3.7',
    'mock>=1.0.1',
    'mock-django==0.6.9',
    'six>=1.9.0',
    'sqlalchemy>=1.0.12',
    'responses',
]

setup(
    name='waldur-mastermind',
    version='3.7.0',
    author='OpenNode Team',
    author_email='info@opennodecloud.com',
    url='http://waldur.com',
    description='Waldur MasterMind is a hybrid cloud orchestrator.',
    license='MIT',
    long_description=open('README.rst').read(),
    package_dir={'': 'src'},
    packages=find_packages('src', exclude=['*.tests', '*.tests.*', 'tests.*', 'tests']),
    install_requires=install_requires,
    extras_require={
        'test': test_requires,
    },
    zip_safe=False,
    entry_points={
        'console_scripts': (
            'waldur = waldur_core.server.manage:main',
        ),
        'waldur_extensions': (
            'waldur_auth_social = waldur_auth_social.extension:AuthSocialExtension',
            'waldur_auth_openid = waldur_auth_openid.extension:WaldurAuthOpenIDExtension',
            'waldur_auth_saml2 = waldur_auth_saml2.extension:SAML2Extension',
            'waldur_auth_valimo = waldur_auth_valimo.extension:AuthValimoExtension',
            'waldur_auth_bcc = waldur_auth_bcc.extension:AuthBCCExtension',
            'waldur_aws = waldur_aws.extension:AWSExtension',
            'waldur_azure = waldur_azure.extension:AzureExtension',
            'waldur_digitalocean = waldur_digitalocean.extension:DigitalOceanExtension',
            'openstack = waldur_openstack.openstack.extension:OpenStackExtension',
            'openstack_tenant = waldur_openstack.openstack_tenant.extension:OpenStackTenantExtension',
            'waldur_cost_planning = waldur_cost_planning.extension:CostPlanningExtension',
            'waldur_freeipa = waldur_freeipa.extension:FreeIPAExtension',
            'waldur_slurm = waldur_slurm.extension:SlurmExtension',
            'waldur_paypal = waldur_paypal.extension:PayPalExtension',
            'waldur_zabbix = waldur_zabbix.extension:ZabbixExtension',
            'waldur_jira = waldur_jira.extension:JiraExtension',
            'waldur_rijkscloud = waldur_rijkscloud.extension:RijkscloudExtension',
            'waldur_common = waldur_ansible.common.extension:AnsibleCommonExtension',
            'waldur_playbook_jobs = waldur_ansible.playbook_jobs.extension:PlaybookJobsExtension',
            'waldur_python_management = waldur_ansible.python_management.extension:PythonManagementExtension',
            'waldur_jupyter_hub_management = waldur_ansible.jupyter_hub_management.extension:JupyterHubManagementExtension',
            'waldur_packages = waldur_mastermind.packages.extension:PackagesExtension',
            'waldur_invoices = waldur_mastermind.invoices.extension:InvoicesExtension',
            'waldur_support = waldur_mastermind.support.extension:SupportExtension',
            'waldur_analytics = waldur_mastermind.analytics.extension:AnalyticsExtension',
            'waldur_billing = waldur_mastermind.billing.extension:BillingExtension',
            'waldur_booking = waldur_mastermind.booking.extension:BookingExtension',
            'waldur_slurm_invoices = waldur_mastermind.slurm_invoices.extension:SlurmInvoicesExtension',
            'waldur_ansible_estimator = waldur_mastermind.ansible_estimator.extension:AnsibleEstimatorExtension',
            'waldur_zabbix_openstack = waldur_mastermind.zabbix_openstack.extension:ZabbixOpenStackExtension',
            'waldur_marketplace = waldur_mastermind.marketplace.extension:MarketplaceExtension',
            'waldur_marketplace_azure = waldur_mastermind.marketplace_azure.extension:MarketplaceAzureExtension',
            'waldur_marketplace_openstack = waldur_mastermind.marketplace_openstack.extension:MarketplaceOpenStackExtension',
            'waldur_marketplace_support = waldur_mastermind.marketplace_support.extension:MarketplaceSupportExtension',
            'waldur_marketplace_slurm = waldur_mastermind.marketplace_slurm.extension:MarketplaceSlurmExtension',
            'waldur_support_invoices = waldur_mastermind.support_invoices.extension:SupportInvoicesExtension',
        ),
    },
    include_package_data=True,
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
    ],
)



from django.conf.urls import url

from . import views


def register_in(router):
    router.register(r'jira', views.JiraServiceViewSet, base_name='jira')
    router.register(r'jira-service-project-link', views.JiraServiceProjectLinkViewSet, base_name='jira-spl')
    router.register(r'jira-attachments', views.AttachmentViewSet, base_name='jira-attachments')
    router.register(r'jira-project-templates', views.ProjectTemplateViewSet, base_name='jira-project-templates')
    router.register(r'jira-projects', views.ProjectViewSet, base_name='jira-projects')
    router.register(r'jira-issue-types', views.IssueTypeViewSet, base_name='jira-issue-types')
    router.register(r'jira-priorities', views.PriorityViewSet, base_name='jira-priorities')
    router.register(r'jira-issues', views.IssueViewSet, base_name='jira-issues')
    router.register(r'jira-comments', views.CommentViewSet, base_name='jira-comments')


urlpatterns = [
    url(r'^api/jira-webhook-receiver/$', views.WebHookReceiverViewSet.as_view(), name='jira-web-hook'),
]



from . import views


def register_in(router):
    router.register(r'digitalocean', views.DigitalOceanServiceViewSet, base_name='digitalocean')
    router.register(r'digitalocean-images', views.ImageViewSet, base_name='digitalocean-image')
    router.register(r'digitalocean-regions', views.RegionViewSet, base_name='digitalocean-region')
    router.register(r'digitalocean-sizes', views.SizeViewSet, base_name='digitalocean-size')
    router.register(r'digitalocean-droplets', views.DropletViewSet, base_name='digitalocean-droplet')
    router.register(r'digitalocean-service-project-link',
                    views.DigitalOceanServiceProjectLinkViewSet, base_name='digitalocean-spl')



import logging
import datetime

from django.db import transaction, IntegrityError
from django.db.models import Q
from django.core import exceptions as django_exceptions

from waldur_core.structure import models as structure_models
from waldur_mastermind.marketplace.plugins import manager
from waldur_mastermind.marketplace import models as marketplace_models
from waldur_mastermind.marketplace_slurm import PLUGIN_NAME
from waldur_mastermind.slurm_invoices import models as slurm_invoices_models
from waldur_slurm.apps import SlurmConfig

logger = logging.getLogger(__name__)


def create_slurm_package(sender, instance, created=False, **kwargs):
    plan = instance.plan

    if not created:
        return

    if plan.offering.type != PLUGIN_NAME:
        return

    if plan.scope:
        return

    if not isinstance(plan.offering.scope, structure_models.ServiceSettings):
        logger.warning('Skipping plan synchronization because offering scope is not service settings. '
                       'Plan ID: %s', plan.id)
        return

    if plan.offering.scope.type != SlurmConfig.service_name:
        logger.warning('Skipping plan synchronization because service settings type is not SLURM. '
                       'Plan ID: %s', plan.id)
        return

    expected_types = set(manager.get_component_types(PLUGIN_NAME))
    actual_types = set(plan.components.values_list('component__type', flat=True))
    if expected_types != actual_types:
        return

    prices = {component.component.type: component.price
              for component in plan.components.all()}

    with transaction.atomic():
        slurm_package = slurm_invoices_models.SlurmPackage.objects.create(
            service_settings=plan.offering.scope,
            name=plan.name,
            product_code=plan.product_code,
            article_code=plan.article_code,
            cpu_price=prices.get('cpu'),
            gpu_price=prices.get('gpu'),
            ram_price=prices.get('ram'),
        )
        plan.scope = slurm_package
        plan.save()


def create_slurm_usage(sender, instance, created=False, **kwargs):
    allocation_usage = instance
    allocation = allocation_usage.allocation

    try:
        resource = marketplace_models.Resource.objects.get(scope=allocation)
    except django_exceptions.ObjectDoesNotExist:
        return

    date = datetime.date(year=allocation_usage.year, month=allocation_usage.month, day=1)

    for component in manager.get_components(PLUGIN_NAME):
        usage = getattr(allocation_usage, component.type + '_usage')

        try:
            plan_component = marketplace_models.OfferingComponent.objects.get(
                offering=resource.offering,
                type=component.type
            )
            plan_period = marketplace_models.ResourcePlanPeriod.objects. \
                filter(Q(start__lte=date) | Q(start__isnull=True)). \
                filter(Q(end__gt=date) | Q(end__isnull=True)). \
                get(resource=resource)
            marketplace_models.ComponentUsage.objects.create(
                resource=resource,
                component=plan_component,
                usage=usage,
                date=date,
                plan_period=plan_period,
            )
        except django_exceptions.ObjectDoesNotExist:
            logger.warning('Skipping AllocationUsage synchronization because this '
                           'marketplace.OfferingComponent does not exist.'
                           'AllocationUsage ID: %s', allocation_usage.id)
        except IntegrityError:
            logger.warning('Skipping AllocationUsage synchronization because this marketplace.ComponentUsage exists.'
                           'AllocationUsage ID: %s', allocation_usage.id)


def update_component_quota(sender, instance, created=False, **kwargs):
    if created:
        return

    allocation = instance

    try:
        resource = marketplace_models.Resource.objects.get(scope=allocation)
    except django_exceptions.ObjectDoesNotExist:
        return

    for component in manager.get_components(PLUGIN_NAME):
        usage = getattr(allocation, component.type + '_usage')
        limit = getattr(allocation, component.type + '_limit')

        try:
            plan_component = marketplace_models.OfferingComponent.objects.get(
                offering=resource.offering,
                type=component.type
            )
            component_quota = marketplace_models.ComponentQuota.objects.get(
                resource=resource,
                component=plan_component,
            )
            component_quota.limit = limit
            component_quota.usage = usage
            component_quota.save()

        except marketplace_models.OfferingComponent.DoesNotExist:
            logger.warning('Skipping Allocation synchronization because this '
                           'marketplace.OfferingComponent does not exist.'
                           'Allocation ID: %s', allocation.id)
        except marketplace_models.ComponentQuota.DoesNotExist:
            marketplace_models.ComponentQuota.objects.create(
                resource=resource,
                component=plan_component,
                limit=limit,
                usage=usage
            )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openstack', '0018_replace_security_group'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='instance',
            name='billing_backend_id',
        ),
        migrations.RemoveField(
            model_name='instance',
            name='last_usage_update_time',
        ),
    ]



import hashlib
import string
import uuid

from django.db import transaction
from django.utils.crypto import get_random_string
from rest_framework import serializers

from waldur_core.core import serializers as core_serializers
from waldur_core.core import utils as core_utils
from waldur_core.core.models import SshPublicKey
from waldur_core.structure import serializers as structure_serializers

from . import models


def hash_string(value, length=16):
    return hashlib.sha256(value).hexdigest()[:length]


def generate_username():
    return 'user{}'.format(core_utils.pwgen(4))


def generate_password():
    lowercase = get_random_string(5, string.ascii_lowercase)
    uppercase = get_random_string(5, string.ascii_uppercase)
    digits = get_random_string(5, string.digits)
    return lowercase + uppercase + digits


class ServiceSerializer(core_serializers.ExtraFieldOptionsMixin,
                        structure_serializers.BaseServiceSerializer):

    SERVICE_ACCOUNT_EXTRA_FIELDS = {
        'tenant_id': 'Azure Active Directory tenant id or domain',
        'client_id': 'Azure Active Directory Application Client ID',
        'client_secret': 'Azure Active Directory Application Secret',
        'subscription_id': 'Azure Subscription Id',
    }

    class Meta(structure_serializers.BaseServiceSerializer.Meta):
        model = models.AzureService
        view_name = 'azure-detail'


class ImageSerializer(structure_serializers.BasePropertySerializer):

    class Meta(object):
        model = models.Image
        view_name = 'azure-image-detail'
        fields = ('url', 'uuid', 'publisher', 'name', 'sku', 'version')
        extra_kwargs = {
            'url': {'lookup_field': 'uuid'},
        }


class SizeSerializer(structure_serializers.BasePropertySerializer):

    class Meta(object):
        model = models.Size
        view_name = 'azure-size-detail'
        fields = ('url', 'uuid', 'name', 'max_data_disk_count',
                  'memory_in_mb', 'number_of_cores', 'os_disk_size_in_mb', 'resource_disk_size_in_mb')
        extra_kwargs = {
            'url': {'lookup_field': 'uuid'},
        }


class LocationSerializer(structure_serializers.BasePropertySerializer):

    class Meta(object):
        model = models.Location
        view_name = 'azure-location-detail'
        fields = ('url', 'uuid', 'name', 'latitude', 'longitude')
        extra_kwargs = {
            'url': {'lookup_field': 'uuid'},
        }


class ServiceProjectLinkSerializer(structure_serializers.BaseServiceProjectLinkSerializer):

    class Meta(structure_serializers.BaseServiceProjectLinkSerializer.Meta):
        model = models.AzureServiceProjectLink
        view_name = 'azure-spl-detail'
        extra_kwargs = {
            'service': {'lookup_field': 'uuid', 'view_name': 'azure-detail'},
        }


class BaseResourceSerializer(structure_serializers.BaseResourceSerializer):
    service = serializers.HyperlinkedRelatedField(
        source='service_project_link.service',
        view_name='azure-detail',
        read_only=True,
        lookup_field='uuid',
    )

    service_project_link = serializers.HyperlinkedRelatedField(
        view_name='azure-spl-detail',
        queryset=models.AzureServiceProjectLink.objects.all(),
        allow_null=True,
        required=False,
    )

    class Meta(structure_serializers.BaseResourceSerializer.Meta):
        protected_fields = structure_serializers.BaseResourceSerializer.Meta.protected_fields + (
            'name',
        )


class ResourceGroupSerializer(BaseResourceSerializer):
    location = serializers.HyperlinkedRelatedField(
        view_name='azure-location-detail',
        lookup_field='uuid',
        queryset=models.Location.objects.all(),
    )

    class Meta(BaseResourceSerializer.Meta):
        model = models.ResourceGroup
        view_name = 'azure-resource-group-detail'
        fields = BaseResourceSerializer.Meta.fields + ('location',)


class BaseResourceGroupSerializer(BaseResourceSerializer):
    resource_group_name = serializers.ReadOnlyField(source='resource_group.name')
    location_name = serializers.ReadOnlyField(source='resource_group.location.name')

    resource_group = serializers.HyperlinkedRelatedField(
        view_name='azure-resource-group-detail',
        lookup_field='uuid',
        read_only=True,
    )

    location = serializers.HyperlinkedRelatedField(
        view_name='azure-location-detail',
        lookup_field='uuid',
        queryset=models.Location.objects.all(),
        write_only=True,
    )


class VirtualMachineSerializer(structure_serializers.VirtualMachineSerializer,
                               BaseResourceGroupSerializer):

    image = serializers.HyperlinkedRelatedField(
        view_name='azure-image-detail',
        lookup_field='uuid',
        queryset=models.Image.objects.all(),
    )

    size = serializers.HyperlinkedRelatedField(
        view_name='azure-size-detail',
        lookup_field='uuid',
        queryset=models.Size.objects.all(),
    )

    ssh_public_key = serializers.HyperlinkedRelatedField(
        view_name='sshpublickey-detail',
        lookup_field='uuid',
        queryset=SshPublicKey.objects.all(),
        required=False,
        allow_null=True,
        write_only=True,
        source='ssh_key'
    )

    size_name = serializers.ReadOnlyField(source='size.name')
    image_name = serializers.ReadOnlyField(source='image.name')

    class Meta(structure_serializers.VirtualMachineSerializer.Meta):
        model = models.VirtualMachine
        view_name = 'azure-virtualmachine-detail'
        fields = structure_serializers.VirtualMachineSerializer.Meta.fields + (
            'image', 'size', 'user_data', 'runtime_state', 'start_time',
            'cores', 'ram', 'disk', 'image_name', 'location',
            'resource_group', 'username', 'password',
            'resource_group_name', 'location_name', 'image_name', 'size_name'
        )
        protected_fields = structure_serializers.VirtualMachineSerializer.Meta.protected_fields + (
            'image', 'size', 'user_data', 'name'
        )
        read_only_fields = structure_serializers.VirtualMachineSerializer.Meta.read_only_fields + (
            'runtime_state', 'start_time', 'cores', 'ram', 'disk', 'image_name',
            'username', 'password'
        )

    @transaction.atomic
    def create(self, validated_data):
        vm_name = validated_data['name']
        spl = validated_data['service_project_link']
        size = validated_data['size']
        location = validated_data.pop('location')

        resource_group_name = 'group{}'.format(uuid.uuid4().hex)
        storage_account_name = 'storage{}'.format(hash_string(vm_name.lower(), 14))
        network_name = 'net{}'.format(vm_name)
        subnet_name = 'subnet{}'.format(vm_name)
        nic_name = 'nic{}'.format(vm_name)
        config_name = 'ipconf{}'.format(vm_name)
        public_ip_name = 'pubip{}'.format(vm_name)
        security_group_name = 'NSG{}'.format(vm_name)

        resource_group = models.ResourceGroup.objects.create(
            service_project_link=spl,
            name=resource_group_name,
            location=location,
        )

        models.StorageAccount.objects.create(
            service_project_link=spl,
            name=storage_account_name,
            resource_group=resource_group,
        )

        network = models.Network.objects.create(
            service_project_link=spl,
            resource_group=resource_group,
            name=network_name,
            cidr='10.0.0.0/16',
        )

        subnet = models.SubNet.objects.create(
            service_project_link=spl,
            resource_group=resource_group,
            name=subnet_name,
            cidr='10.0.0.0/24',
            network=network,
        )

        public_ip = models.PublicIP.objects.create(
            service_project_link=spl,
            resource_group=resource_group,
            location=location,
            name=public_ip_name,
        )

        security_group = models.SecurityGroup.objects.create(
            service_project_link=spl,
            resource_group=resource_group,
            name=security_group_name,
        )

        nic = models.NetworkInterface.objects.create(
            service_project_link=spl,
            resource_group=resource_group,
            name=nic_name,
            subnet=subnet,
            config_name=config_name,
            public_ip=public_ip,
            security_group=security_group,
        )

        validated_data['ram'] = size.memory_in_mb
        validated_data['cores'] = size.number_of_cores
        validated_data['disk'] = size.os_disk_size_in_mb + size.resource_disk_size_in_mb

        validated_data['network_interface'] = nic
        validated_data['resource_group'] = resource_group
        validated_data['username'] = generate_username()
        validated_data['password'] = generate_password()

        return super(VirtualMachineSerializer, self).create(validated_data)


class PublicIPSerializer(BaseResourceSerializer):

    location = serializers.HyperlinkedRelatedField(
        view_name='azure-location-detail',
        lookup_field='uuid',
        queryset=models.Location.objects.all(),
    )

    resource_group = serializers.HyperlinkedRelatedField(
        view_name='azure-resource-group-detail',
        lookup_field='uuid',
        queryset=models.ResourceGroup.objects.all(),
    )

    class Meta(BaseResourceSerializer.Meta):
        model = models.PublicIP
        view_name = 'azure-public-ip-detail'
        fields = BaseResourceSerializer.Meta.fields + (
            'location', 'resource_group',
        )


class SQLServerSerializer(BaseResourceGroupSerializer):
    class Meta(BaseResourceGroupSerializer.Meta):
        model = models.SQLServer
        view_name = 'azure-sql-server-detail'
        fields = BaseResourceGroupSerializer.Meta.fields + (
            'resource_group', 'location', 'username', 'password', 'storage_mb',
            'username', 'password', 'fqdn',
            'resource_group_name', 'location_name'
        )
        read_only_fields = BaseResourceGroupSerializer.Meta.read_only_fields + (
            'username', 'password', 'fqdn'
        )

    @transaction.atomic
    def create(self, validated_data):
        spl = validated_data['service_project_link']
        location = validated_data.pop('location')

        resource_group_name = 'group{}'.format(uuid.uuid4().hex)

        resource_group = models.ResourceGroup.objects.create(
            service_project_link=spl,
            name=resource_group_name,
            location=location,
        )
        validated_data['resource_group'] = resource_group
        validated_data['username'] = generate_username()
        validated_data['password'] = generate_password()

        return super(SQLServerSerializer, self).create(validated_data)


class SQLDatabaseSerializer(BaseResourceSerializer):
    resource_group_name = serializers.ReadOnlyField(source='server.resource_group.name')
    location_name = serializers.ReadOnlyField(source='server.resource_group.location.name')
    server_name = serializers.ReadOnlyField(source='server.name')
    server_uuid = serializers.ReadOnlyField(source='server.uuid')

    server = serializers.HyperlinkedRelatedField(
        view_name='azure-sql-server-detail',
        lookup_field='uuid',
        queryset=models.SQLServer.objects.all(),
    )

    class Meta(BaseResourceSerializer.Meta):
        model = models.SQLDatabase
        view_name = 'azure-sql-database-detail'
        fields = BaseResourceSerializer.Meta.fields + (
            'server', 'charset', 'collation',
            'resource_group_name', 'location_name',
            'server_name', 'server_uuid',
        )


class SQLDatabaseCreateSerializer(serializers.ModelSerializer):
    class Meta(object):
        model = models.SQLDatabase
        fields = ('name', 'description')

    def create(self, validated_data):
        server = self.context['view'].get_object()
        validated_data['server'] = server
        validated_data['service_project_link'] = server.service_project_link
        return super(SQLDatabaseCreateSerializer, self).create(validated_data)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('cost_tracking', '0012_fix_fields_length'),
    ]

    operations = [
        migrations.AlterField(
            model_name='defaultpricelistitem',
            name='item_type',
            field=models.CharField(max_length=255),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='pricelistitem',
            name='item_type',
            field=models.CharField(max_length=255),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-03-12 11:17
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0060_plan_backend_id'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='ProjectResourceCount',
            new_name='AggregateResourceCount',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.13 on 2018-08-21 08:55
from __future__ import unicode_literals

from django.db import migrations
import django_fsm


class Migration(migrations.Migration):

    dependencies = [
        ('marketplace', '0020_offering_vendor_details'),
    ]

    operations = [
        migrations.AlterField(
            model_name='order',
            name='state',
            field=django_fsm.FSMIntegerField(choices=[(1, 'requested for approval'), (2, 'executing'), (3, 'done'), (4, 'terminated'), (5, 'erred'), (6, 'rejected')], default=1),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-03-12 11:20
from __future__ import unicode_literals

from django.db import migrations


def add_scope(apps, schema_editor):
    ContentType = apps.get_model('contenttypes', 'ContentType')
    Project = apps.get_model('structure', 'Project')
    AggregateResourceCount = apps.get_model('marketplace', 'aggregateresourcecount')

    project_ct = ContentType.objects.get_for_model(Project)

    for row in AggregateResourceCount.objects.all():
        row.content_type = project_ct
        row.object_id = row.project_id
        row.save()


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('marketplace', '0062_add_scope_to_resource_count'),
    ]

    operations = [
        migrations.RunPython(add_scope)
    ]



from django.core.management.base import BaseCommand

from waldur_mastermind.support import backend as support_backend

from ... import models


class Command(BaseCommand):
    help = ("Backend data update if a server was switched.")

    def handle(self, *args, **options):
        models.RequestType.objects.all().delete()
        backend = support_backend.get_active_backend()
        backend.pull_request_types()

        for support_customer in models.SupportCustomer.objects.all():
            exists_user = backend.manager.search_users(support_customer.user.email)

            if exists_user:
                backend_user = exists_user[0]
            else:
                backend_user = backend.create_user(support_customer.user)

            support_customer.backend_id = backend_user.key
            support_customer.save()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-27 12:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0035_offering_plan'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='offeringtemplate',
            options={'ordering': ['sort_order', 'name']},
        ),
        migrations.AddField(
            model_name='offeringtemplate',
            name='sort_order',
            field=models.PositiveSmallIntegerField(default=1),
        ),
    ]



from django.conf import settings as django_settings
from rest_framework import exceptions

from waldur_core.structure import permissions as structure_permissions


def can_register_service_provider(request, customer):
    if request.user.is_staff:
        return
    if not django_settings.WALDUR_MARKETPLACE['OWNER_CAN_REGISTER_SERVICE_PROVIDER']:
        raise exceptions.PermissionDenied()
    structure_permissions.is_owner(request, None, customer)


def check_permissions_for_state_change(request, view, order=None):
    if not order:
        return

    user = request.user
    if user_can_approve_order(user, order.project):
        return

    raise exceptions.PermissionDenied()


def user_can_approve_order(user, project):
    if user.is_staff:
        return True

    if django_settings.WALDUR_MARKETPLACE['OWNER_CAN_APPROVE_ORDER'] and \
            structure_permissions._has_owner_access(user, project.customer):
        return True

    if django_settings.WALDUR_MARKETPLACE['MANAGER_CAN_APPROVE_ORDER'] and \
            structure_permissions._has_manager_access(user, project):
        return True

    if django_settings.WALDUR_MARKETPLACE['ADMIN_CAN_APPROVE_ORDER'] and \
            structure_permissions._has_admin_access(user, project):
        return True

    return False



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-07-23 13:21
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import waldur_core.core.fields


class Migration(migrations.Migration):

    dependencies = [
        ('logging', '0002_immutable_default_json'),
    ]

    operations = [
        migrations.AddField(
            model_name='systemnotification',
            name='name',
            field=models.CharField(default='', max_length=150, verbose_name='name'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='systemnotification',
            name='roles',
            field=waldur_core.core.fields.JSONField(default=list, verbose_name='List of roles'),
        ),
        migrations.AlterField(
            model_name='systemnotification',
            name='hook_content_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='contenttypes.ContentType'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0005_add_user_language_and_competence'),
    ]

    operations = [
        migrations.AddField(
            model_name='user',
            name='is_support',
            field=models.BooleanField(default=False, help_text='Designates whether the user is a global support user.', verbose_name='support status'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('packages', '0002_openstack_packages'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='openstackpackage',
            options={'verbose_name': 'OpenStack VPC package', 'verbose_name_plural': 'OpenStack VPC packages'},
        ),
        migrations.AlterModelOptions(
            name='packagetemplate',
            options={'verbose_name': 'VPC package template', 'verbose_name_plural': 'VPC package templates'},
        ),
    ]



from django.contrib.contenttypes.models import ContentType
from django.utils import timezone

from waldur_core.cost_tracking import ConsumableItem
from waldur_core.cost_tracking.models import DefaultPriceListItem
from waldur_mastermind.invoices import models as invoices_models
from waldur_mastermind.packages import models as packages_models
from waldur_openstack.openstack import models as openstack_models


class Types(object):
    PACKAGE_TEMPLATE = 'PackageTemplate'


def get_consumable_item(package_template):
    return ConsumableItem(
        item_type=Types.PACKAGE_TEMPLATE,
        key=package_template.name,
        default_price=package_template.price / 24)  # default price per hour


def sync_price_list_item(package_template):
    resource_content_type = ContentType.objects.get_for_model(openstack_models.Tenant)
    consumable_item = get_consumable_item(package_template)
    DefaultPriceListItem._create_or_update_default_price_list_item(
        resource_content_type=resource_content_type,
        consumable_item=consumable_item,
    )


def get_openstack_items():
    model_type = ContentType.objects.get_for_model(packages_models.OpenStackPackage)
    return invoices_models.GenericInvoiceItem.objects.filter(content_type=model_type)


def shift_backward(item, days=1):
    """
    Shifts end date to N 'days' ago.
    If N is larger than it lasts - zero length will be set.
    :param days: number of days to shift end date
    """
    if (item.end - item.start).days > days:
        end = item.end - timezone.timedelta(days=1)
    else:
        end = item.start

    item.end = end
    item.save()


def extend_to_the_end_of_the_day(item):
    item.end = item.end.replace(hour=23, minute=59, second=59)
    item.save()


def get_invoice_item_name(package):
    template_category = package.template.get_category_display()
    tenant_name = package.tenant.name
    template_name = package.template.name

    if template_category:
        return '%s (%s / %s)' % (tenant_name, template_category, template_name)
    else:
        return '%s (%s)' % (tenant_name, template_name)



from __future__ import unicode_literals

from . import views


def register_in(router):
    router.register(r'zabbix-openstack-links', views.LinkViewSet, base_name='zabbix-openstack-links')


urlpatterns = []



import datetime
import logging

from django.contrib.contenttypes.models import ContentType
from django.core.management.base import CommandError
from django.db import transaction

from waldur_core.core.utils import DryRunCommand
from waldur_mastermind.marketplace import models as marketplace_models
from waldur_mastermind.marketplace.utils import format_list
from waldur_slurm import models as slurm_models

logger = logging.getLogger(__name__)


class ImportAllocationException(Exception):
    pass


@transaction.atomic()
def import_allocation(dry_run=False):
    ct = ContentType.objects.get_for_model(slurm_models.Allocation)
    exist_ids = marketplace_models.Resource.objects.filter(content_type=ct).values_list('object_id', flat=True)
    missing_allocations = slurm_models.Allocation.objects.exclude(id__in=exist_ids)

    if dry_run:
        logger.warning('Allocations would be imported to marketplace. '
                       'IDs: %s.' % format_list(missing_allocations))
        return missing_allocations.count()

    for allocation in missing_allocations:
        offering = marketplace_models.Offering.objects.filter(scope=allocation.service_settings).first()
        component_cpu = offering.components.get(type='cpu')
        component_gpu = offering.components.get(type='gpu')
        component_ram = offering.components.get(type='ram')

        try:
            plan = marketplace_models.Plan.objects.get(offering=offering)
        except marketplace_models.Plan.DoesNotExist:
            logger.warning('Resource has not been created. Because Plan is not found. '
                           'Offering UUID: %s.' % offering.uuid.hex)
            continue

        state = marketplace_models.Resource.States.OK if allocation.is_active \
            else marketplace_models.Resource.States.TERMINATED
        resource = marketplace_models.Resource.objects.create(
            content_type=ct,
            object_id=allocation.id,
            state=state,
            project=allocation.service_project_link.project,
            offering=offering,
            created=allocation.created,
            plan=plan,
            limits={'deposit_limit': int(allocation.deposit_limit)}
        )

        marketplace_models.ComponentQuota.objects.create(
            resource=resource,
            limit=allocation.cpu_limit,
            component=component_cpu)
        marketplace_models.ComponentQuota.objects.create(
            resource=resource,
            limit=allocation.gpu_limit,
            component=component_gpu)
        marketplace_models.ComponentQuota.objects.create(
            resource=resource,
            limit=allocation.ram_limit,
            component=component_ram)

        resource_plan_period = marketplace_models.ResourcePlanPeriod.objects.create(
            resource=resource,
            plan=resource.plan,
            start=allocation.created,
            end=None,
        )

        for allocation_usage in allocation.allocationusage_set.all():
            if allocation_usage.year == allocation.created.year and \
                    allocation_usage.month == allocation.created.month:
                day = allocation.created.day
            else:
                day = 1

            date = datetime.date(allocation_usage.year, allocation_usage.month, day)
            marketplace_models.ComponentUsage.objects.create(
                resource=resource,
                component=component_cpu,
                usage=allocation_usage.cpu_usage,
                date=date,
                plan_period=resource_plan_period
            )
            marketplace_models.ComponentUsage.objects.create(
                resource=resource,
                component=component_gpu,
                usage=allocation_usage.gpu_usage,
                date=date,
                plan_period=resource_plan_period
            )
            marketplace_models.ComponentUsage.objects.create(
                resource=resource,
                component=component_ram,
                usage=allocation_usage.ram_usage,
                date=date,
                plan_period=resource_plan_period
            )
    return missing_allocations.count()


class Command(DryRunCommand):
    help = """Import existing SLURM allocations as marketplace resources.
    It is expected that offerings for SLURM service settings are imported before this command is ran."""

    def handle(self, dry_run, *args, **options):
        try:
            allocation_counter = import_allocation(dry_run)
            self.stdout.write(self.style.SUCCESS('%s resources have been created.' % allocation_counter))
        except ImportAllocationException as e:
            raise CommandError(e.message)



from __future__ import unicode_literals

import os

from croniter import croniter
from django.core.exceptions import ValidationError
from django.core.validators import BaseValidator, URLValidator
from django.utils import timezone
from django.utils.deconstruct import deconstructible
from django.utils.translation import ugettext_lazy as _
from iptools.ipv4 import validate_cidr as is_valid_ipv4_cidr
from iptools.ipv6 import validate_cidr as is_valid_ipv6_cidr

from waldur_core.core import exceptions
from waldur_core.core import magic


def validate_cron_schedule(value):
    try:
        base_time = timezone.now()
        croniter(value, base_time)
    except (KeyError, ValueError) as e:
        raise ValidationError(str(e))


@deconstructible
class MinCronValueValidator(BaseValidator):
    """
    Validate that the period of cron schedule is greater than or equal to provided limit_value in hours,
    otherwise raise ValidationError.
    """
    message = _('Ensure schedule period is greater than or equal to %(limit_value)s hour(s).')
    code = 'min_cron_value'

    def compare(self, cleaned, limit_value):
        validate_cron_schedule(cleaned)

        now = timezone.now()
        schedule = croniter(cleaned, now)
        closest_schedule = schedule.get_next(timezone.datetime)
        next_schedule = schedule.get_next(timezone.datetime)
        schedule_interval_in_hours = (next_schedule - closest_schedule).total_seconds() / 3600
        return schedule_interval_in_hours < limit_value


def validate_name(value):
    if len(value.strip()) == 0:
        raise ValidationError(_('Ensure that name has at least one non-whitespace character.'))


class StateValidator(object):

    def __init__(self, *valid_states):
        self.valid_states = valid_states

    def __call__(self, resource):
        if resource.state not in self.valid_states:
            states_names = dict(resource.States.CHOICES)
            valid_states_names = [str(states_names[state]) for state in self.valid_states]
            raise exceptions.IncorrectStateException(_('Valid states for operation: %s.') % ', '.join(valid_states_names))


class RuntimeStateValidator(StateValidator):

    def __call__(self, resource):
        if resource.runtime_state not in self.valid_states:
            raise exceptions.IncorrectStateException(_('Valid runtime states for operation: %s.') % ', '.join(self.valid_states))


class BackendURLValidator(URLValidator):
    schemes = ['ldap', 'ldaps', 'http', 'https']


def is_valid_ipv46_cidr(value):
    return is_valid_ipv6_cidr(value) or is_valid_ipv4_cidr(value)


def validate_cidr_list(value):
    if not value.strip():
        return
    invalid_items = []
    for item in value.split(','):
        item = item.strip()
        if not is_valid_ipv46_cidr(item):
            invalid_items.append(item)
    if invalid_items:
        raise ValidationError(
            message=_('The following items are invalid: %s'),
            code='invalid',
            params=', '.join(invalid_items),
        )


# max bytes to read for file type detection
READ_SIZE = 5 * (1024 * 1024)   # 5MB


# Based on https://github.com/mckinseyacademy/django-upload-validator/blob/master/upload_validator/__init__.py
@deconstructible
class FileTypeValidator(object):
    """
    File type validator for validating mimetypes and extensions
    Args:
        allowed_types (list): list of acceptable mimetypes e.g; ['image/jpeg', 'application/pdf']
                    see https://www.iana.org/assignments/media-types/media-types.xhtml
        allowed_extensions (list, optional): list of allowed file extensions e.g; ['.jpeg', '.pdf', '.docx']
    """
    type_message = _(
        "File type '%(detected_type)s' is not allowed. "
        "Allowed types are: '%(allowed_types)s'."
    )

    extension_message = _(
        "File extension '%(extension)s' is not allowed. "
        "Allowed extensions are: '%(allowed_extensions)s'."
    )

    def __init__(self, allowed_types, allowed_extensions=()):
        self.allowed_mimes = allowed_types
        self.allowed_exts = allowed_extensions

    def __call__(self, fileobj):
        detected_type = magic.from_buffer(fileobj.read(READ_SIZE), mime=True)
        root, extension = os.path.splitext(fileobj.name.lower())

        # seek back to start so a valid file could be read
        # later without resetting the position
        fileobj.seek(0)

        # some versions of libmagic do not report proper mimes for Office subtypes
        # use detection details to transform it to proper mime
        if detected_type in ('application/octet-stream', 'application/vnd.ms-office'):
            detected_type = self.check_word_or_excel(fileobj, detected_type, extension)

        if detected_type not in self.allowed_mimes:
            # use more readable file type names for feedback message
            allowed_types = map(lambda mime_type: mime_type.split('/')[1], self.allowed_mimes)

            raise ValidationError(
                message=self.type_message,
                params={
                    'detected_type': detected_type,
                    'allowed_types': ', '.join(allowed_types)
                },
                code='invalid_type'
            )

        if self.allowed_exts and (extension not in self.allowed_exts):
            raise ValidationError(
                message=self.extension_message,
                params={
                    'extension': extension,
                    'allowed_extensions': ', '.join(self.allowed_exts)
                },
                code='invalid_extension'
            )

    def check_word_or_excel(self, fileobj, detected_type, extension):
        """
        Returns proper mimetype in case of word or excel files
        """
        word_strings = ['Microsoft Word', 'Microsoft Office Word', 'Microsoft Macintosh Word']
        excel_strings = ['Microsoft Excel', 'Microsoft Office Excel', 'Microsoft Macintosh Excel']
        office_strings = ['Microsoft OOXML']

        file_type_details = magic.from_buffer(fileobj.read(READ_SIZE))

        fileobj.seek(0)

        if any(string in file_type_details for string in word_strings):
            detected_type = 'application/msword'
        elif any(string in file_type_details for string in excel_strings):
            detected_type = 'application/vnd.ms-excel'
        elif any(string in file_type_details for string in office_strings) or \
                (detected_type == 'application/vnd.ms-office'):
            if extension in ('.doc', '.docx'):
                detected_type = 'application/msword'
            if extension in ('.xls', '.xlsx'):
                detected_type = 'application/vnd.ms-excel'

        return detected_type


ImageValidator = FileTypeValidator(
    allowed_types=[
        'image/png',
        'image/jpeg',
        'image/svg',
        'image/svg+xml',
        'image/x-icon',
    ]
)


@deconstructible
class BlacklistValidator(object):
    message = _('This value is blacklisted.')
    code = 'blacklist'
    blacklist = ()

    def __init__(self, blacklist=None, message=None, code=None):
        if blacklist is not None:
            self.blacklist = blacklist
        if message is not None:
            self.message = message
        if code is not None:
            self.code = code

    def __call__(self, value):
        if value in self.blacklist:
            raise ValidationError(self.message, code=self.code)



from __future__ import unicode_literals

from django.apps import AppConfig
from django.db.models import signals
from django_fsm import signals as fsm_signals


class ZabbixConfig(AppConfig):
    name = 'waldur_zabbix'
    verbose_name = 'Waldur Zabbix'
    service_name = 'Zabbix'

    def ready(self):
        from waldur_core.structure import SupportedServices, models as structure_models
        # structure
        from .backend import ZabbixBackend
        SupportedServices.register_backend(ZabbixBackend)

        from . import handlers
        for index, resource_model in enumerate(structure_models.ResourceMixin.get_all_models()):

            fsm_signals.post_transition.connect(
                handlers.delete_hosts_on_scope_deletion,
                sender=resource_model,
                dispatch_uid='waldur_zabbix.handlers.delete_hosts_on_scope_deletion_%s_%s' % (
                    index, resource_model.__name__)
            )

        signals.post_save.connect(
            handlers.refresh_database_connection,
            sender=structure_models.ServiceSettings,
            dispatch_uid='waldur_zabbix.handlers.refresh_database_connection',
        )



from rest_framework import serializers


class UserDetailRequestSerializer(serializers.Serializer):
    civil_number = serializers.RegexField(r'^\d+$')
    tax_number = serializers.RegexField(r'^\d+-\d+$')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.utils.timezone
import model_utils.fields
import django.db.models.deletion
from django.conf import settings
import waldur_core.core.fields


class Migration(migrations.Migration):

    #replaces = [('logging', '0001_initial'), ('logging', '0002_alert_acknowledged'), ('logging', '0003_emailhook_webhook')]

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Alert',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False)),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('alert_type', models.CharField(max_length=50)),
                ('message', models.CharField(max_length=255)),
                ('severity', models.SmallIntegerField(choices=[(10, 'Debug'), (20, 'Info'), (30, 'Warning'), (40, 'Error')])),
                ('closed', models.DateTimeField(null=True, blank=True)),
                ('context', waldur_core.core.fields.JSONField(blank=True)),
                ('object_id', models.PositiveIntegerField(null=True)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.SET_NULL, to='contenttypes.ContentType', null=True)),
                ('acknowledged', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='EmailHook',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False)),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('event_types', waldur_core.core.fields.JSONField(verbose_name='List of event types')),
                ('is_active', models.BooleanField(default=True)),
                ('last_published', models.DateTimeField(default=django.utils.timezone.now)),
                ('email', models.EmailField(max_length=75)),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='WebHook',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, verbose_name='created', editable=False)),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, verbose_name='modified', editable=False)),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('event_types', waldur_core.core.fields.JSONField(verbose_name='List of event types')),
                ('is_active', models.BooleanField(default=True)),
                ('last_published', models.DateTimeField(default=django.utils.timezone.now)),
                ('destination_url', models.URLField()),
                ('content_type', models.SmallIntegerField(default=1, choices=[(1, 'json'), (2, 'form')])),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
            bases=(models.Model,),
        ),
    ]



import collections
import logging

Component = collections.namedtuple('Component', ('type', 'name', 'measured_unit', 'billing_type'))
logger = logging.getLogger(__name__)


class PluginManager(object):
    def __init__(self):
        self.backends = {}

    def register(self, offering_type,
                 create_resource_processor,
                 update_resource_processor=None,
                 delete_resource_processor=None,
                 components=None,
                 service_type=None,
                 can_terminate_order_item=False,
                 secret_attributes=None):
        """

        :param offering_type: string which consists of application name and model name,
                              for example Support.OfferingTemplate
        :param create_resource_processor: class which receives order item
        :param update_resource_processor: class which receives order item
        :param delete_resource_processor: class which receives order item
        :param components: tuple available plan components, for example
                           Component(type='storage', name='Storage', measured_unit='GB')
        :param service_type: optional string indicates service type to be used
        :param can_terminate_order_item: optional boolean indicates whether order item can be terminated
        :param secret_attributes: optional list of strings each of which corresponds to secret attribute key,
        for example, VPC username and password.
        """
        self.backends[offering_type] = {
            'create_resource_processor': create_resource_processor,
            'update_resource_processor': update_resource_processor,
            'delete_resource_processor': delete_resource_processor,
            'components': components,
            'service_type': service_type,
            'can_terminate_order_item': can_terminate_order_item,
            'secret_attributes': secret_attributes,
        }

    def get_offering_types(self):
        """
        Return list of offering types.
        """
        return self.backends.keys()

    def get_service_type(self, offering_type):
        """
        Return a service type for given offering_type.
        :param offering_type: offering type name
        :return: string or None
        """
        return self.backends.get(offering_type, {}).get('service_type')

    def get_components(self, offering_type):
        """
        Return a list of components for given offering_type.
        :param offering_type: offering type name
        :return: list of components
        """
        return self.backends.get(offering_type, {}).get('components') or []

    def get_component_types(self, offering_type):
        """
        Return a components types for given offering_type.
        :param offering_type: offering type name
        :return: set of component types
        """
        return {component.type for component in self.get_components(offering_type)}

    def can_terminate_order_item(self, offering_type):
        """
        Returns true if order item can be terminated.
        """
        return self.backends.get(offering_type, {}).get('can_terminate_order_item')

    def get_secret_attributes(self, offering_type):
        """
        Returns list of secret attributes for given offering type.
        """
        secret_attributes = self.backends.get(offering_type, {}).get('secret_attributes')
        if callable(secret_attributes):
            secret_attributes = secret_attributes()
        return secret_attributes or []

    def get_processor(self, offering_type, processor_type):
        """
        Return a processor class for given offering type and order item type.
        """
        return self.backends.get(offering_type, {}).get(processor_type)


manager = PluginManager()



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2018-01-22 13:01
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('invoices', '0018_genericinvoiceitem_quantity'),
    ]

    operations = [
        migrations.RenameField(
            model_name='paymentdetails',
            old_name='account',
            new_name='_account',
        ),
        migrations.RenameField(
            model_name='paymentdetails',
            old_name='address',
            new_name='_address',
        ),
        migrations.RenameField(
            model_name='paymentdetails',
            old_name='bank',
            new_name='_bank',
        ),
        migrations.RenameField(
            model_name='paymentdetails',
            old_name='company',
            new_name='_company',
        ),
        migrations.RenameField(
            model_name='paymentdetails',
            old_name='country',
            new_name='_country',
        ),
        migrations.RenameField(
            model_name='paymentdetails',
            old_name='email',
            new_name='_email',
        ),
        migrations.RenameField(
            model_name='paymentdetails',
            old_name='phone',
            new_name='_phone',
        ),
        migrations.RenameField(
            model_name='paymentdetails',
            old_name='postal',
            new_name='_postal',
        ),
        migrations.RenameField(
            model_name='paymentdetails',
            old_name='type',
            new_name='_type',
        ),
        migrations.RenameField(
            model_name='paymentdetails',
            old_name='accounting_start_date',
            new_name='_accounting_start_date',
        ),
        migrations.RenameField(
            model_name='paymentdetails',
            old_name='default_tax_percent',
            new_name='_default_tax_percent',
        ),
        migrations.AlterField(
            model_name='paymentdetails',
            name='_account',
            field=models.CharField(blank=True, editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name='paymentdetails',
            name='_address',
            field=models.CharField(blank=True, editable=False, max_length=300),
        ),
        migrations.AlterField(
            model_name='paymentdetails',
            name='_bank',
            field=models.CharField(blank=True, editable=False, max_length=150),
        ),
        migrations.AlterField(
            model_name='paymentdetails',
            name='_company',
            field=models.CharField(blank=True, editable=False, max_length=150),
        ),
        migrations.AlterField(
            model_name='paymentdetails',
            name='_country',
            field=models.CharField(blank=True, editable=False, max_length=50),
        ),
        migrations.AlterField(
            model_name='paymentdetails',
            name='_email',
            field=models.EmailField(blank=True, editable=False, max_length=75),
        ),
        migrations.AlterField(
            model_name='paymentdetails',
            name='_phone',
            field=models.CharField(blank=True, editable=False, max_length=20),
        ),
        migrations.AlterField(
            model_name='paymentdetails',
            name='_postal',
            field=models.CharField(blank=True, editable=False, max_length=20),
        ),
        migrations.AlterField(
            model_name='paymentdetails',
            name='_type',
            field=models.CharField(blank=True, editable=False, max_length=150),
        ),
        migrations.AlterField(
            model_name='paymentdetails',
            name='_accounting_start_date',
            field=models.DateTimeField(blank=True, default=django.utils.timezone.now, editable=False,
                                       verbose_name='Start date of accounting'),
        ),
        migrations.AlterField(
            model_name='paymentdetails',
            name='_default_tax_percent',
            field=models.DecimalField(blank=True, decimal_places=2, default=0, editable=False, max_digits=4,
                                      validators=[django.core.validators.MinValueValidator(0),
                                                  django.core.validators.MaxValueValidator(100)]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.16 on 2018-12-28 13:46
from __future__ import unicode_literals

from django.db import migrations, models
import django_fsm


class Migration(migrations.Migration):

    dependencies = [
        ('support', '0032_requesttype'),
    ]

    operations = [
        migrations.CreateModel(
            name='IssueStatus',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, help_text='Status name in Jira.', unique=True)),
                ('type', django_fsm.FSMIntegerField(choices=[(0, 'Resolved'), (1, 'Canceled')], default=0)),
            ],
            options={
                'verbose_name': 'Issue status',
                'verbose_name_plural': 'Issue statuses',
            },
        ),
    ]



from rest_framework import serializers

from waldur_mastermind.marketplace import processors
from waldur_mastermind.marketplace.plugins import manager
from waldur_mastermind.marketplace_slurm import PLUGIN_NAME
from waldur_slurm import models as slurm_models
from waldur_slurm import views as slurm_views


class CreateAllocationProcessor(processors.BaseCreateResourceProcessor):
    viewset = slurm_views.AllocationViewSet

    def get_post_data(self):
        if not self.order_item.limits:
            raise serializers.ValidationError('Quota are not defined.')

        spl_url = processors.get_spl_url(slurm_models.SlurmServiceProjectLink, self.order_item)
        payload = dict(
            name=self.order_item.offering.name,
            service_project_link=spl_url,
        )

        for component_type in manager.get_component_types(PLUGIN_NAME):
            try:
                limit = self.order_item.limits[component_type]
            except KeyError:
                raise serializers.ValidationError('%s component quota is not defined' % component_type)
            else:
                payload[component_type + '_limit'] = limit
        return payload


class DeleteAllocationProcessor(processors.DeleteResourceProcessor):
    viewset = slurm_views.AllocationViewSet
